@@ -19,6 +19,23 @@
  * Furthermore, John Morrissey gives permission to link this program with
  * OpenSSL, and distribute the resulting executable, without including the
  * source code for OpenSSL in the source distribution.
+ *
+ * As of mod_ldap 2.8.10, you MUST send me a postcard at the following
+ * address if you use mod_ldap. If you do not send a postcard, you are in
+ * violation of mod_ldap's licensing. I'm not trying to be needlessly
+ * uptight about this, all it boils down to is that I want to know that
+ * people are using mod_ldap and that they appreciate the effort I put into
+ * working on it (and providing free support, I might add). I tried making
+ * this optional on some of my other software, but it's been several months
+ * and I've yet to receive even a single postcard. If you don't have(?) or
+ * don't want to send a postcard, send a photograph of your local area or of
+ * something geographically close to you that you find interesting. It'll
+ * only cost you a dollar or two. Please do me a favor and send it.
+ *
+ * John Morrissey
+ * P.O. Box 209
+ * Henrietta, NY 14534-2638
+ * USA
  */
 
 /*
@@ -43,7 +60,7 @@
  *                                                   LDAPDefaultAuthScheme
  *
  * 
- * $Id: mod_ldap.c,v 1.25 2002/08/31 00:45:02 jwm Exp $
+ * $Id: mod_ldap.c,v 1.28 2002/12/05 21:16:47 castaglia Exp $
  * $Libraries: -lldap -llber$
  */
 
@@ -250,9 +267,9 @@ pr_ldap_mkdir(char *dir, mode_t mode, uid_t uid, gid_t gid)
   mode_t old_umask;
   struct stat st;
 
-  if (fs_stat(dir, &st) == -1) {
+  if (pr_fsio_stat(dir, &st) == -1) {
     if (errno != ENOENT) {
-      log_pri(LOG_WARNING, "mod_ldap: pr_ldap_mkdir(): unable to create directory %s: fs_stat() failed: %s", dir, strerror(errno));
+      log_pri(LOG_WARNING, "mod_ldap: pr_ldap_mkdir(): unable to create directory %s: pr_fsio_stat() failed: %s", dir, strerror(errno));
       return;
     }
   }
@@ -262,12 +279,12 @@ pr_ldap_mkdir(char *dir, mode_t mode, uid_t uid, gid_t gid)
   old_umask = umask(0);
 
   PRIVS_ROOT;
-  if (mkdir(dir, mode) != 0) {
+  if (pr_fsio_mkdir(dir, mode) != 0) {
     PRIVS_RELINQUISH;
     log_pri(LOG_WARNING, "mod_ldap: pr_ldap_mkdir(): unable to create directory %s: %s", dir, strerror(errno));
     return;
   }
-  if (fs_chown(dir, uid, gid) == -1) {
+  if (pr_fsio_chown(dir, uid, gid) == -1) {
     PRIVS_RELINQUISH;
     log_pri(LOG_WARNING, "mod_ldap: pr_ldap_mkdir(): unable to chown directory %s: %s", dir, strerror(errno));
     return;
@@ -285,7 +302,7 @@ pr_ldap_mkpath(pool *p, const char *path, mode_t mode)
 
   /* If the full path already exists, just return without bothering to
      stat() its individual components. */
-  if (fs_stat(path, &st) != -1)
+  if (pr_fsio_stat(path, &st) != -1)
     return 0;
 
   buf = pstrdup(p, path);
@@ -948,7 +965,7 @@ handle_ldap_is_auth(cmd_rec *cmd)
       return DECLINED(cmd); /* Can't find the user in the LDAP directory. */
 
   if (!ldap_authbinds && !pw->pw_passwd)
-    return ERROR_INT(cmd, AUTH_NOPWD);
+    return ERROR_INT(cmd, PR_AUTH_NOPWD);
 
   /* FIXME: If we pass a "" or NULL "crypted password" argument to
             auth_check, the mod_unixpw auth handler gets called before the
@@ -958,7 +975,7 @@ handle_ldap_is_auth(cmd_rec *cmd)
             name escapes me right now) For now, we'll kludge around this by
             passing "*", which mod_unixpw will happily deny auth to. */
   if (auth_check(cmd->tmp_pool, ldap_authbinds ? "*" : pw->pw_passwd, username, cmd->argv[1]))
-    return ERROR_INT(cmd, AUTH_BADPWD);
+    return ERROR_INT(cmd, PR_AUTH_BADPWD);
 
   create_homedir(cmd->tmp_pool, pw->pw_dir);
   return HANDLED(cmd);
