@@ -124,14 +124,14 @@ _make_cmd (pool * cp, int argc, ...)
   int i;
 
   pool *newpool = NULL;
-  newpool = make_named_sub_pool( cp, "temp pool" );
+  newpool = make_sub_pool( cp );
   c = pcalloc(newpool, sizeof(cmd_rec));
 
   c->pool = newpool;
   c->argv = pcalloc(newpool, sizeof(void *) * (argc + 1));
 
   c->argc = argc;
-  c->symtable_index = -1;
+  c->stash_index = -1;
 
   c->argv[0] = MOD_RATIO_VERSION;
   va_start (args, argc);
@@ -148,13 +148,13 @@ _dispatch_ratio (cmd_rec * cmd, char *match)
   authtable *m;
   modret_t *mr = NULL;
 
-  m = mod_find_auth_symbol (match, &cmd->symtable_index, NULL);
+  m = pr_stash_get_symbol (PR_SYM_AUTH, match, NULL, &cmd->stash_index);
   while (m)
     {
       mr = call_module_auth (m->m, m->handler, cmd);
       if (MODRET_ISHANDLED (mr) || MODRET_ISERROR (mr))
 	break;
-      m = mod_find_auth_symbol (match, &cmd->symtable_index, m);
+      m = pr_stash_get_symbol (PR_SYM_AUTH, match, m, &cmd->stash_index);
     }
   if (MODRET_ISERROR (mr))
     log_debug (DEBUG0, "Aiee! mod_ratio internal!  %s", MODRET_ERRMSG (mr));
@@ -400,8 +400,8 @@ pre_cmd_retr (cmd_rec * cmd)
 
   if (stats.frate && stats.files < 1)
     {
-      add_response_err (R_550, "%s", g.filemsg);
-      add_response_err (R_550,
+      pr_response_add_err (R_550, "%s", g.filemsg);
+      pr_response_add_err (R_550,
 			"%s: FILE RATIO: %s  Down: %i  Up: only %i!",
 			cmd->arg, stats.ftext, stats.fretr, stats.fstor);
       return ERROR (cmd);
@@ -412,13 +412,13 @@ pre_cmd_retr (cmd_rec * cmd)
       path = dir_realpath (cmd->tmp_pool, cmd->arg);
       if (path
 	  && dir_check (cmd->tmp_pool, cmd->argv[0], cmd->group, path, NULL)
-	  && fs_stat (path, &sbuf) > -1)
+	  && pr_fsio_stat (path, &sbuf) > -1)
 	fsize = sbuf.st_size;
 
       if ((stats.bytes - (fsize / 1024)) < 0)
 	{
-	  add_response_err (R_550, "%s", g.bytemsg);
-	  add_response_err (R_550,
+	  pr_response_add_err (R_550, "%s", g.bytemsg);
+	  pr_response_add_err (R_550,
 			    "%s: BYTE RATIO: %s  Down: %imb  Up: only %imb!",
 	cmd->arg, stats.btext, (stats.bretr / 1024), (stats.bstor / 1024));
 	  return ERROR (cmd);
@@ -471,7 +471,7 @@ cmd_cwd (cmd_rec * cmd)
 	{
 	  if (!*((char *) c->argv[0]))
 	    return DECLINED (cmd);
-	  add_response (R_250, "%s", c->argv[0]);
+	  pr_response_add (R_250, "%s", (char *)c->argv[0]);
 	  c = find_config_next (c, c->next, CONF_PARAM, "CwdRatioMsg", FALSE);
 	}
     }
@@ -497,6 +497,11 @@ ratio_cmd (cmd_rec * cmd)
 	 }
 		   	     }
 
+   if (session.anon_user)
+     sstrncpy(g.user, session.anon_user, sizeof(g.user));
+   if (strlen(g.user) == 0)
+     strcpy(g.user, "NOBODY\0");
+
    if (!gotratuser && !fileerr && g.save) {
 	usrfile=fopen(g.ratiofile,"r");
         while (fgets(usrstr,sizeof(usrstr),usrfile)!=NULL) {
@@ -573,14 +578,14 @@ ratio_cmd (cmd_rec * cmd)
 
       if (RATIO_ENFORCE)
 	{
-	  add_response (r, "%s%s%s", sbuf1, sbuf2, sbuf3);
+	  pr_response_add (r, "%s%s%s", sbuf1, sbuf2, sbuf3);
 	  if (stats.frate && stats.files < 0)
-	    add_response (r, "%s", g.filemsg);
+	    pr_response_add (r, "%s", g.filemsg);
 	  if (stats.brate && stats.bytes < 0)
-	    add_response (r, "%s", g.bytemsg);
+	    pr_response_add (r, "%s", g.bytemsg);
 	}
       else
-	add_response (r, "%s%s%s", sbuf1, g.leechmsg ? "  " : "", g.leechmsg);
+	pr_response_add (r, "%s%s%s", sbuf1, g.leechmsg ? "  " : "", g.leechmsg);
     }
   return DECLINED (cmd);
 }
@@ -596,24 +601,24 @@ cmd_site (cmd_rec * cmd)
   if(!strcasecmp(cmd->argv[1], "RATIO")) {
     _calc_ratios(cmd);
     snprintf(buf, sizeof(buf), RATIO_STUFFS);
-    add_response(R_214, "Current Ratio: ( %s )", buf);
+    pr_response_add(R_214, "Current Ratio: ( %s )", buf);
     if(stats.frate)
-      add_response(R_214,
+      pr_response_add(R_214,
 		   "Files: %s  Down: %i  Up: %i  CR: %i file%s",
 		   stats.ftext, stats.fretr, stats.fstor,
 		   stats.files, (stats.files != 1) ? "s" : "");
     if(stats.brate)
-      add_response(R_214,
+      pr_response_add(R_214,
 		   "Bytes: %s  Down: %imb  Up: %imb  CR: %i Mbytes",
 		   stats.btext, (stats.bretr / 1024), (stats.bstor / 1024),
-		   (stats.bytes / 1024), stats.bytes);
+		   (stats.bytes / 1024));
     return HANDLED(cmd);
   }
   
   if(!strcasecmp (cmd->argv[1], "HELP")) {
-    add_response(R_214,
+    pr_response_add(R_214,
 		 "The following SITE extensions are recognized:");
-    add_response(R_214, "RATIO " "-- show all ratios in effect");
+    pr_response_add(R_214, "RATIO " "-- show all ratios in effect");
   }
   
   return DECLINED (cmd);
