@@ -17,10 +17,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
  *
- * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
- * and other respective copyright holders give permission to link this program
- * with OpenSSL, and distribute the resulting executable, without including
- * the source code for OpenSSL in the source distribution.
+ * As a special exemption, Andrew Houghton and other respective copyright
+ * holders give permission to link this program with OpenSSL, and distribute
+ * the resulting executable, without including the source code for OpenSSL in
+ * the source distribution.
  */
 
 /*
@@ -255,8 +255,8 @@ static void _sql_check_cmd(cmd_rec *cmd, char *msg)
   if ((!cmd) || (!cmd->tmp_pool)) {
     log_pri(PR_LOG_ERR, _MOD_VERSION ": '%s' was passed an invalid cmd_rec. "
 	    "Shutting down.", msg);
-    log_debug(DEBUG_WARN, _MOD_VERSION ": '%s' was passed an invalid cmd_rec. "
-	      "Shutting down.", msg);
+    sql_log(DEBUG_WARN, "'%s' was passed an invalid cmd_rec. Shutting down.",
+      msg);
     end_login(1);
   }    
 
@@ -278,8 +278,7 @@ static int _sql_timer_callback(CALLBACK_FRAME)
     entry = ((conn_entry_t **) conn_cache->elts)[cnt];
 
     if (entry->timer == p2) {
-      log_debug(DEBUG_INFO, _MOD_VERSION ": timer expired for connection '%s'",
-		entry->name);
+      sql_log(DEBUG_INFO, "timer expired for connection '%s'", entry->name);
       cmd = _sql_make_cmd( conn_pool, 2, entry->name, "1" );
       cmd_close( cmd );
       _sql_free_cmd( cmd );
@@ -349,7 +348,7 @@ static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
   sql_data_t *sd = NULL;
   char **data = NULL;
   unsigned long cnt = 0;
-  unsigned long index = 0;
+  unsigned long i = 0;
 
   if (!conn) 
     return PR_ERR_SQL_BADCMD(cmd);
@@ -374,7 +373,7 @@ static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
   
   while ((row = mysql_fetch_row( result ))) {
     for (cnt = 0; cnt < sd->fnum; cnt++)
-      data[index++] = pstrdup(cmd->tmp_pool, row[cnt]);
+      data[i++] = pstrdup(cmd->tmp_pool, row[cnt]);
   }
   
   /* at this point either we finished correctly or an error occurred in the
@@ -387,7 +386,7 @@ static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
   }
 
   mysql_free_result( result );
-  data[index] = NULL;
+  data[i] = NULL;
   sd->data = data;
 
   return mod_create_data( cmd, (void *) sd );
@@ -415,19 +414,19 @@ MODRET cmd_open(cmd_rec *cmd)
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_open");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_open");
 
   _sql_check_cmd(cmd, "cmd_open" );
 
   if (cmd->argc < 1) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_open");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
     return PR_ERR_SQL_BADCMD(cmd);
   }    
 
   /* get the named connection */
 
   if (!(entry = _sql_get_connection( cmd->argv[0]))) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_open");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
     return PR_ERR_SQL_UNDEF(cmd);
   } 
 
@@ -441,9 +440,9 @@ MODRET cmd_open(cmd_rec *cmd)
     if (entry->timer) {
       reset_timer( entry->timer, &sql_mysql_module );
     }
-    log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' count is now %d",
-	      entry->name, entry->connections);
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_open");
+    sql_log(DEBUG_INFO, "connection '%s' count is now %d", entry->name,
+      entry->connections);
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
     return HANDLED(cmd);
   }
 
@@ -453,8 +452,8 @@ MODRET cmd_open(cmd_rec *cmd)
   if (!conn->mysql) {
     log_pri(PR_LOG_ERR, _MOD_VERSION ": failed to allocate memory for "
 	    "MYSQL structure.  Shutting down.");
-    log_debug(DEBUG_WARN, _MOD_VERSION ": failed to allocate memory for "
-	      "MYSQL structure.  Shutting down.");
+    sql_log(DEBUG_WARN, "%s", "failed to allocate memory for MYSQL structure. "
+      "Shutting down.");
     end_login(1);
   }
 
@@ -463,7 +462,7 @@ MODRET cmd_open(cmd_rec *cmd)
 			  (int) strtol(conn->port, (char **) NULL, 10), 
 			  NULL, 0)) {
     /* if it didn't work, return an error */
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_open");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
     return _build_error( cmd, conn );
   }
 
@@ -475,9 +474,8 @@ MODRET cmd_open(cmd_rec *cmd)
     entry->timer = add_timer(entry->ttl, -1, 
 			     &sql_mysql_module, 
 			     _sql_timer_callback);
-    log_debug(DEBUG_INFO,
-	      _MOD_VERSION ": connection '%s' - %d second timer started",
-	      entry->name, entry->ttl);
+    sql_log(DEBUG_INFO, "connection '%s' - %d second timer started",
+      entry->name, entry->ttl);
 
     /* timed connections get re-bumped so they don't go away when cmd_close
      * is called.
@@ -486,13 +484,12 @@ MODRET cmd_open(cmd_rec *cmd)
   }
 
   /* return HANDLED */
-  log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' opened",
-	    entry->name);
+  sql_log(DEBUG_INFO, "connection '%s' opened", entry->name);
 
-  log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' count is now %d",
-	    entry->name, entry->connections);
+  sql_log(DEBUG_INFO, ": connection '%s' count is now %d", entry->name,
+    entry->connections);
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_open");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
   return HANDLED(cmd);
 }
 
@@ -525,18 +522,18 @@ MODRET cmd_close(cmd_rec *cmd)
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_close");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_close");
 
   _sql_check_cmd(cmd, "cmd_close");
 
   if ((cmd->argc < 1) || (cmd->argc > 2)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_close");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_close");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection */
   if (!(entry = _sql_get_connection( cmd->argv[0] ))) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_close");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_close");
     return PR_ERR_SQL_UNDEF(cmd);
   }
 
@@ -544,10 +541,10 @@ MODRET cmd_close(cmd_rec *cmd)
 
   /* if we're closed already (connections == 0) return HANDLED */
   if (entry->connections == 0) {
-    log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' count is now %d",
-	      entry->name, entry->connections);
+    sql_log(DEBUG_INFO, "connection '%s' count is now %d", entry->name,
+      entry->connections);
 
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_close");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_close");
     return HANDLED(cmd);
   }
 
@@ -563,17 +560,15 @@ MODRET cmd_close(cmd_rec *cmd)
     if (entry->timer) {
       remove_timer( entry->timer, &sql_mysql_module );
       entry->timer = 0;
-      log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' - timer stopped",
-		entry->name );
+      sql_log(DEBUG_INFO, "connection '%s' - timer stopped", entry->name);
     }
 
-    log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' closed",
-	      entry->name);
+    sql_log(DEBUG_INFO, "connection '%s' closed", entry->name);
   }
 
-  log_debug(DEBUG_INFO, _MOD_VERSION ": connection '%s' count is now %d",
-	    entry->name, entry->connections);
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_close");
+  sql_log(DEBUG_INFO, "connection '%s' count is now %d", entry->name,
+    entry->connections);
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_close");
   
   return HANDLED(cmd);
 }
@@ -616,12 +611,12 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL; 
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_defineconnection");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_defineconnection");
 
   _sql_check_cmd(cmd, "cmd_defineconnection");
 
   if ((cmd->argc < 4) || (cmd->argc > 5) || (!cmd->argv[0])) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_defineconnection");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_defineconnection");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
@@ -666,7 +661,7 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
 
   /* insert the new conn_info into the connection hash */
   if (!(entry = _sql_add_connection(conn_pool, name, (void *) conn))) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_defineconnection");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_defineconnection");
     return PR_ERR_SQL_REDEF(cmd);
   }
 
@@ -678,14 +673,14 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
   entry->timer = 0;
   entry->connections = 0;
 
-  log_debug(DEBUG_INFO, _MOD_VERSION ":  name: '%s'", entry->name);
-  log_debug(DEBUG_INFO, _MOD_VERSION ":  user: '%s'", conn->user);
-  log_debug(DEBUG_INFO, _MOD_VERSION ":  host: '%s'", conn->host);
-  log_debug(DEBUG_INFO, _MOD_VERSION ":    db: '%s'", conn->db);
-  log_debug(DEBUG_INFO, _MOD_VERSION ":  port: '%s'", conn->port);
-  log_debug(DEBUG_INFO, _MOD_VERSION ":   ttl: '%d'", entry->ttl);
+  sql_log(DEBUG_INFO, " name: '%s'", entry->name);
+  sql_log(DEBUG_INFO, " user: '%s'", conn->user);
+  sql_log(DEBUG_INFO, " host: '%s'", conn->host);
+  sql_log(DEBUG_INFO, "   db: '%s'", conn->db);
+  sql_log(DEBUG_INFO, " port: '%s'", conn->port);
+  sql_log(DEBUG_INFO, "  ttl: '%d'", entry->ttl);
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_defineconnection");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_defineconnection");
   return HANDLED(cmd);
 }
 
@@ -745,19 +740,19 @@ MODRET cmd_select(cmd_rec *cmd)
   int cnt = 0;
   cmd_rec *close_cmd;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_select");
+  sql_log(DEBUG_FUNC, "%s", ": entering \tmysql cmd_select");
 
   _sql_check_cmd(cmd, "cmd_select");
 
   if (cmd->argc < 2) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_select");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_select");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
     return PR_ERR_SQL_UNDEF(cmd);
   }
   
@@ -765,7 +760,7 @@ MODRET cmd_select(cmd_rec *cmd)
 
   cmr = cmd_open(cmd);
   if (MODRET_ERROR(cmr)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_select");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
     return cmr;
   }
 
@@ -798,7 +793,7 @@ MODRET cmd_select(cmd_rec *cmd)
   }
 
   /* log the query string */
-  log_debug( DEBUG_INFO, _MOD_VERSION ": query \"%s\"", query);
+  sql_log( DEBUG_INFO, "query \"%s\"", query);
 
   /* perform the query.  if it doesn't work, log the error, close the
    * connection then return the error from the query processing.
@@ -810,7 +805,7 @@ MODRET cmd_select(cmd_rec *cmd)
     cmd_close(close_cmd);
     _sql_free_cmd( close_cmd );
 
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_select");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
     return dmr;
   }
 
@@ -819,7 +814,7 @@ MODRET cmd_select(cmd_rec *cmd)
    */
   dmr = _build_data( cmd, conn );
   if (MODRET_ERROR(dmr)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_select");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
@@ -833,8 +828,7 @@ MODRET cmd_select(cmd_rec *cmd)
   cmd_close(close_cmd);
   _sql_free_cmd( close_cmd );
 
-
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_select");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
   return dmr;
 }
 
@@ -878,19 +872,19 @@ MODRET cmd_insert(cmd_rec *cmd)
   char *query = NULL;
   cmd_rec *close_cmd;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_insert");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_insert");
 
   _sql_check_cmd(cmd, "cmd_insert");
 
   if ((cmd->argc != 2) && (cmd->argc != 4)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_insert");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_insert");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
     return PR_ERR_SQL_UNDEF(cmd);
   }
 
@@ -898,7 +892,7 @@ MODRET cmd_insert(cmd_rec *cmd)
 
   cmr = cmd_open(cmd);
   if (MODRET_ERROR(cmr)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_insert");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
     return cmr;
   }
 
@@ -912,7 +906,7 @@ MODRET cmd_insert(cmd_rec *cmd)
   }
 
   /* log the query string */
-  log_debug( DEBUG_INFO, _MOD_VERSION ": query \"%s\"", query);
+  sql_log(DEBUG_INFO, "query \"%s\"", query);
 
   /* perform the query.  if it doesn't work, log the error, close the
    * connection (and log any errors there, too) then return the error
@@ -925,7 +919,7 @@ MODRET cmd_insert(cmd_rec *cmd)
     cmd_close(close_cmd);
     _sql_free_cmd( close_cmd );
 
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_insert");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
     return dmr;
   }
 
@@ -934,7 +928,7 @@ MODRET cmd_insert(cmd_rec *cmd)
   cmd_close(close_cmd);
   _sql_free_cmd( close_cmd );
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_insert");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
   return HANDLED(cmd);
 }
 
@@ -977,19 +971,19 @@ MODRET cmd_update(cmd_rec *cmd)
   char *query = NULL;
   cmd_rec *close_cmd;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_update");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_update");
 
   _sql_check_cmd(cmd, "cmd_update");
 
   if ((cmd->argc < 2) || (cmd->argc > 4)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_update");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_update");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
     return PR_ERR_SQL_UNDEF(cmd);
   }
 
@@ -997,7 +991,7 @@ MODRET cmd_update(cmd_rec *cmd)
 
   cmr = cmd_open(cmd);
   if (MODRET_ERROR(cmr)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_update");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
     return cmr;
   }
 
@@ -1012,7 +1006,7 @@ MODRET cmd_update(cmd_rec *cmd)
   }
 
   /* log the query string */
-  log_debug( DEBUG_INFO, _MOD_VERSION ": query \"%s\"", query);
+  sql_log(DEBUG_INFO, "query \"%s\"", query);
 
   /* perform the query.  if it doesn't work close the connection, then
    * return the error from the query processing.
@@ -1024,7 +1018,7 @@ MODRET cmd_update(cmd_rec *cmd)
     cmd_close(close_cmd);
     _sql_free_cmd( close_cmd );
 
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_update");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
     return dmr;
   }
 
@@ -1033,7 +1027,7 @@ MODRET cmd_update(cmd_rec *cmd)
   cmd_close(close_cmd);
   _sql_free_cmd( close_cmd );
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_update");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
   return HANDLED(cmd);
 }
 
@@ -1057,18 +1051,18 @@ MODRET cmd_update(cmd_rec *cmd)
  */
 MODRET cmd_procedure(cmd_rec *cmd)
 {
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_procedure");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_procedure");
 
   _sql_check_cmd(cmd, "cmd_procedure");
 
   if (cmd->argc != 3) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_procedure");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_procedure");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* MySQL does not support procedures.  Nothing to do. */
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_procedure");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_procedure");
 
   return ERROR_MSG(cmd, _MOD_VERSION, "backend does not support procedures");
 }
@@ -1101,19 +1095,19 @@ MODRET cmd_query(cmd_rec *cmd)
   char *query = NULL;
   cmd_rec *close_cmd;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_query");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_query");
 
   _sql_check_cmd(cmd, "cmd_query");
 
   if (cmd->argc != 2) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_query");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_query");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
     return PR_ERR_SQL_UNDEF(cmd);
   }
 
@@ -1121,14 +1115,14 @@ MODRET cmd_query(cmd_rec *cmd)
 
   cmr = cmd_open(cmd);
   if (MODRET_ERROR(cmr)) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_query");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
     return cmr;
   }
 
   query = pstrcat(cmd->tmp_pool, cmd->argv[1], NULL);
 
   /* log the query string */
-  log_debug( DEBUG_INFO, _MOD_VERSION ": query \"%s\"", query);
+  sql_log(DEBUG_INFO, "query \"%s\"", query);
 
   /* perform the query.  if it doesn't work close the connection, then
    * return the error from the query processing.
@@ -1140,7 +1134,7 @@ MODRET cmd_query(cmd_rec *cmd)
     cmd_close(close_cmd);
     _sql_free_cmd( close_cmd );
     
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_query");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
     return dmr;
   }
 
@@ -1151,7 +1145,7 @@ MODRET cmd_query(cmd_rec *cmd)
   if ( mysql_field_count( conn->mysql ) ) {
     dmr = _build_data( cmd, conn );
     if (MODRET_ERROR(dmr)) {
-      log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_query");
+      sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
     }
   } else {
     dmr = HANDLED(cmd);
@@ -1162,7 +1156,7 @@ MODRET cmd_query(cmd_rec *cmd)
   cmd_close(close_cmd);
   _sql_free_cmd( close_cmd );
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_query");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
   return dmr;
 }
 
@@ -1198,19 +1192,19 @@ MODRET cmd_escapestring(cmd_rec * cmd)
   char *unescaped = NULL;
   char *escaped = NULL;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_escapestring");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_escapestring");
 
   _sql_check_cmd(cmd, "cmd_escapestring");
 
   if (cmd->argc != 2) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_escapestring");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_escapestring");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_escapestring");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_escapestring");
     return PR_ERR_SQL_UNDEF(cmd);
   }
 
@@ -1231,7 +1225,7 @@ MODRET cmd_escapestring(cmd_rec * cmd)
   mysql_escape_string( escaped, unescaped, strlen(unescaped) );
 #endif
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_escapestring");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_escapestring");
   return mod_create_data(cmd, (void *) escaped);
 }
 
@@ -1246,12 +1240,12 @@ MODRET cmd_escapestring(cmd_rec * cmd)
  *  cmd->argv[2]: hashed string
  *
  * Returns:
- *  HANDLED(cmd)                   -- passwords match
- *  ERROR_INT(cmd,AUTH_NOPWD)      -- missing password
- *  ERROR_INT(cmd,AUTH_BADPWD)     -- passwords don't match
- *  ERROR_INT(cmd,AUTH_DISABLEPWD) -- password is disabled
- *  ERROR_INT(cmd,AUTH_AGEPWD)     -- password is aged
- *  ERROR(cmd)                     -- unknown error
+ *  HANDLED(cmd)                        -- passwords match
+ *  ERROR_INT(cmd, PR_AUTH_NOPWD)       -- missing password
+ *  ERROR_INT(cmd, PR_AUTH_BADPWD)      -- passwords don't match
+ *  ERROR_INT(cmd, PR_AUTH_DISABLEPWD) -- password is disabled
+ *  ERROR_INT(cmd, PR_AUTH_AGEPWD)      -- password is aged
+ *  ERROR(cmd)                          -- unknown error
  *
  * Notes:
  *  If this backend does not provide this functionality, this cmd *must*
@@ -1266,25 +1260,25 @@ MODRET cmd_checkauth(cmd_rec * cmd)
   char *c_hash = NULL;
   int success = 0;
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": entering \tcmd_checkauth");
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_checkauth");
 
   _sql_check_cmd(cmd, "cmd_checkauth");
 
   if (cmd->argc != 3) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_checkauth");
+    sql_log(DEBUG_FUNC, "exiting \tmysql cmd_checkauth");
     return PR_ERR_SQL_BADCMD(cmd);
   }
 
   /* get the named connection -- not used in this case, but for consistency */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_checkauth");
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_checkauth");
     return PR_ERR_SQL_UNDEF(cmd);
   }
 
   if (cmd->argv[1] == NULL) {
-    log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_checkauth");
-    return ERROR_INT(cmd, AUTH_NOPWD);
+    sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_checkauth");
+    return ERROR_INT(cmd, PR_AUTH_NOPWD);
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1296,9 +1290,9 @@ MODRET cmd_checkauth(cmd_rec * cmd)
 
   success = !strcmp(scrambled, c_hash); 
 
-  log_debug(DEBUG_FUNC, _MOD_VERSION ": exiting \tcmd_checkauth");
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_checkauth");
 
-  return success ? HANDLED(cmd) : ERROR_INT(cmd, AUTH_BADPWD);
+  return success ? HANDLED(cmd) : ERROR_INT(cmd, PR_AUTH_BADPWD);
 }
 
 /*
@@ -1364,13 +1358,12 @@ cmdtable sql_cmdtable[] = {
  * sql_mysql_init: Used to initialize the connection cache and register
  *  the exit handler.
  */
-static int sql_mysql_init()
-{
-  conn_pool  = make_named_sub_pool(session.pool,"connections");
+static int sql_mysql_init(void) {
+  conn_pool  = make_sub_pool(session.pool);
   conn_cache = make_array(session.pool, DEF_CONN_POOL_SIZE,
 			  sizeof(conn_entry_t));
 
-  add_exit_handler( _sql_shutdown );
+  pr_exit_register_handler( _sql_shutdown );
 
   return 0;
 }
