@@ -2,8 +2,8 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -25,11 +25,15 @@
  */
 
 /* Generic configuration and standard header file includes.
- * $Id: conf.h,v 1.16 2002/07/24 22:20:20 castaglia Exp $
+ * $Id: conf.h,v 1.31 2003/02/25 18:56:39 castaglia Exp $
  */
 
-#ifndef __CONF_H
-#define __CONF_H
+#ifndef PR_CONF_H
+#define PR_CONF_H
+
+#ifndef _GNU_SOURCE
+# define _GNU_SOURCE
+#endif
 
 #include "version.h"
 #include "config.h"
@@ -48,6 +52,10 @@
 #include <pwd.h>
 #include <grp.h>
 
+#ifdef HAVE_STROPTS_H
+# include <stropts.h>
+#endif
+
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
@@ -215,6 +223,15 @@ char *strchr(),*strrchr();
 # define timer_t p_timer_t
 #endif
 
+/* AIX, when compiled using -D_NO_PROTO, lacks some prototypes without
+ * which ProFTPD may do some funny (and not good) things.  Provide the
+ * prototypes as necessary here.
+ */
+
+#if defined(AIX4) && defined(_NO_PROTO)
+off_t lseek(int, off_t, int);
+#endif
+
 /* See if we have bcopy, if not define them to use the memcpy functions */
 
 #ifndef HAVE_BCOPY
@@ -282,7 +299,7 @@ extern char *alloca();
 
 #endif
 
-#if defined(_FILE_OFFSET_BITS) || defined(_LARGE_FILES) || SIZEOF_UNSIGNED_LONG_LONG == 8
+#if defined(_FILE_OFFSET_BITS) || defined(_LARGE_FILES) || SIZEOF_OFF_T == 8
 # define PR_LU	"llu"
 #else
 # define PR_LU	"lu"
@@ -309,16 +326,22 @@ typedef struct in_addr p_in_addr_t;
 #include "support.h"
 #include "sets.h"
 #include "dirtree.h"
-#include "io.h"
+#include "netio.h"
 #include "modules.h"
+#include "response.h"
 #include "timers.h"
 #include "inet.h"
+#include "bindings.h"
 #include "ident.h"
+#include "feat.h"
 #include "ftp.h"
 #include "log.h"
+#include "scoreboard.h"
 #include "data.h"
 #include "libsupp.h"
-#include "fs.h"
+#include "fsio.h"
+#include "mkhome.h"
+#include "pr-syslog.h"
 
 # ifdef HAVE_SETPASSENT
 #  define setpwent()	setpassent(1)
@@ -330,4 +353,4 @@ typedef struct in_addr p_in_addr_t;
 
 #endif /* __PROFTPD_SUPPORT_LIBRARY */
 
-#endif /* __CONF_H */
+#endif /* PR_CONF_H */
