@@ -2,8 +2,8 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -25,24 +25,24 @@
  */
 
 /* Data connection management prototypes
- * $Id: data.h,v 1.10 2002/06/23 19:03:21 castaglia Exp $
+ * $Id: data.h,v 1.14 2003/01/02 18:25:17 castaglia Exp $
  */
 
-#ifndef __DATACONN_H
-#define __DATACONN_H
+#ifndef PR_DATACONN_H
+#define PR_DATACONN_H
 
-void data_init(char *, int);
-void data_cleanup(void);
-int data_open(char *, char *, int, off_t);
-void data_close(int);
-void data_abort(int, int);
-int data_xfer(char *, int);
-void data_reset(void);
+void pr_data_init(char *, int);
+void pr_data_cleanup(void);
+int pr_data_open(char *, char *, int, off_t);
+void pr_data_close(int);
+void pr_data_abort(int, int);
+int pr_data_xfer(char *, int);
+void pr_data_reset(void);
 
 #ifdef HAVE_SENDFILE
 typedef
 
-#if defined(HAVE_LINUX_SENDFILE) || defined(HAVE_HPUX_SENDFILE)
+#if defined(HAVE_LINUX_SENDFILE) || defined(HAVE_HPUX_SENDFILE) || defined(HAVE_AIX_SENDFILE)
 ssize_t
 #elif defined(HAVE_BSD_SENDFILE)
 off_t
@@ -52,7 +52,7 @@ off_t
 
 pr_sendfile_t;
 
-pr_sendfile_t data_sendfile(int retr_fd, off_t *offset, size_t count);
+pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count);
 #endif /* HAVE_SENDFILE */
 
-#endif /* __DATACONN_H */
+#endif /* PR_DATACONN_H */
