@@ -1,9 +1,9 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -27,11 +27,11 @@
 /*
  * Configuration structure, server, command and associated prototypes.
  *
- * $Id: dirtree.h,v 1.19 2002/08/01 22:08:54 castaglia Exp $
+ * $Id: dirtree.h,v 1.34 2003/02/26 04:30:06 castaglia Exp $
  */
 
-#ifndef __DIRTREE_H
-#define __DIRTREE_H
+#ifndef PR_DIRTREE_H
+#define PR_DIRTREE_H
 
 #include "pool.h"
 #include "sets.h"
@@ -56,9 +56,6 @@ typedef struct server_struc {
   int tcp_swin_override;	/* Specifically override tcp swin */
 
   char *ServerAdmin;		/* Administrator's name */
-  char *ServerMessage;		/* Server's welcome message */
-
-  int AnonymousGreeting;	/* Do not greet until after user login */  
 
   p_in_addr_t *ipaddr;		/* Internal address of this server */
   struct conn_struc *listen;	/* Our listening connection */
@@ -83,7 +80,7 @@ typedef struct cmd_struc {
   char *group;			/* Command grouping */
 
   int  class;			/* The command class */
-  int  symtable_index;		/* hack to speed up symbol hashing in modules.c */
+  int  stash_index;		/* hack to speed up symbol hashing in modules.c */
   privdata_t *private;		/* Private data for passing/retaining between handlers */
   array_header *privarr;	/* Internal use */
 } cmd_rec;
@@ -99,7 +96,7 @@ struct config_struc {
   void **argv;
   long overrides;		/* Override classes */
   long flags;			/* Flags */
-  
+
   server_rec *server;		/* Server this config element is attached to */
   config_rec *parent;		/* Our parent configuration record */
   xaset_t *subset;		/* Sub-configuration */
@@ -112,6 +109,7 @@ struct config_struc {
 #define CONF_VIRTUAL		(1 << 4) /* Virtual host */
 #define CONF_DYNDIR		(1 << 5) /* .ftpaccess file */
 #define CONF_GLOBAL		(1 << 6) /* "Global" context (applies to main server and ALL virtualhosts */
+#define CONF_NAMED		(1 << 7)  /* Named virtual host */
 #define CONF_USERDATA		(1 << 14) /* Runtime user data */
 #define CONF_PARAM		(1 << 15) /* config/args pair */
 
@@ -145,7 +143,6 @@ extern int			SocketBindTight;
 extern char			ServerType;
 extern int			ServerMaxInstances;
 extern int			ServerUseReverseDNS;
-extern int			TimeoutLogin;
 extern int			TimeoutIdle;
 extern int			TimeoutNoXfer;
 extern int			TimeoutStalled;
@@ -162,23 +159,23 @@ extern int			TimeoutStalled;
 
 #define CHECK_HASARGS(x, n)	((x)->argc - 1) == n
 
-#define CHECK_CONF(x,p)		if(!check_conf((x),(p))) \
+#define CHECK_CONF(x,p)		if (!check_context((x),(p))) \
 				CONF_ERROR((x), \
 				pstrcat((x)->tmp_pool,"directive not allowed in ", \
 				get_context_name((x)), \
 				" context",NULL))
 
-#define CHECK_CMD_ARGS(x, n)	if((x)->argc != (n)) { \
-				  add_response_err(R_501, \
-					"Invalid number of arguments."); \
-				  return ERROR((x)); \
-				}
+#define CHECK_CMD_ARGS(x, n)	\
+  if ((x)->argc != (n)) { \
+    pr_response_add_err(R_501, "Invalid number of arguments."); \
+    return ERROR((x)); \
+  }
 
-#define CHECK_CMD_MIN_ARGS(x, n)	if((x)->argc < (n)) { \
-					  add_response_err(R_501, \
-					     "Invalid number of arguments."); \
-					  return ERROR((x)); \
-					}
+#define CHECK_CMD_MIN_ARGS(x, n)	\
+  if ((x)->argc < (n)) { \
+    pr_response_add_err(R_501, "Invalid number of arguments."); \
+    return ERROR((x)); \
+  }
 
 /* Prototypes */
 
@@ -189,7 +186,7 @@ extern int			TimeoutStalled;
 void kludge_disable_umask(void);
 void kludge_enable_umask(void);
 
-int define_exists(const char *);
+unsigned char define_exists(const char *);
 void init_config(void);
 void fixup_servers(void);
 void set_config_stream(FILE *, unsigned int);
@@ -207,15 +204,20 @@ void find_config_set_top(config_rec *);
 int remove_config(xaset_t *, const char *, int);
 class_t *find_class(p_in_addr_t *, char *);
 
-array_header *parse_group_expression(pool *, int *, char **);
-array_header *parse_user_expression(pool *, int *, char **);
-int group_expression(char **);
-int user_expression(char **);
+array_header *pr_parse_expression(pool *, int *, char **);
+unsigned char pr_class_and_expression(char **);
+unsigned char pr_class_or_expression(char **);
+unsigned char pr_group_and_expression(char **);
+unsigned char pr_group_or_expression(char **);
+unsigned char pr_user_and_expression(char **);
+unsigned char pr_user_or_expression(char **);
 
 long get_param_int(xaset_t *, const char *, int);
 long get_param_int_next(const char *, int);
 void *get_param_ptr(xaset_t *, const char *, int);
 void *get_param_ptr_next(const char *, int);
+xaset_t *get_dir_ctxt(char *);
+
 void init_conf_stacks(void);
 void free_conf_stacks(void);
 server_rec *start_new_server(const char *);
@@ -223,13 +225,13 @@ server_rec *end_new_server(void);
 config_rec *start_sub_config(const char *);
 config_rec *end_sub_config(void);
 char *get_word(char **);
-char *get_line(char *, int, FILE *, unsigned int *);
 
 config_rec *dir_match_path(pool *, char *);
-void build_dyn_config(pool *, char *, struct stat *, int);
-int dir_check_hidden(const char *);
+void build_dyn_config(pool *, char *, struct stat *, unsigned char);
+unsigned char dir_hide_file(const char *);
 int dir_check_op_mode(pool *, char *, int, uid_t, gid_t, mode_t);
 int dir_check_full(pool *, char *, char *, char *, int *);
+int dir_check_limits(config_rec *, char *, int);
 int dir_check(pool *, char *, char *, char *, int *);
 int dir_check_canon(pool *, char *, char *, char *, int *);
 int is_dotdir(const char *);
@@ -237,10 +239,10 @@ int login_check_limits(xaset_t *, int, int, int *);
 void resolve_anonymous_dirs(xaset_t *);
 void resolve_defered_dirs(server_rec *);
 void fixup_dirs(server_rec *, int);
-unsigned char check_conf(cmd_rec *, int);
+unsigned char check_context(cmd_rec *, int);
 char *get_context_name(cmd_rec *);
 int get_boolean(cmd_rec *, int);
 char *get_full_cmd(cmd_rec *);
 int match_ip(p_in_addr_t *, char *, const char *);
 
-#endif /* __DIRTREE_H */
+#endif /* PR_DIRTREE_H */
