@@ -2,8 +2,8 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -22,11 +22,13 @@
  * and other respective copyright holders give permission to link this program
  * with OpenSSL, and distribute the resulting executable, without including
  * the source code for OpenSSL in the source distribution.
+ *
+ * $Id: ident.h,v 1.9 2003/02/12 08:49:14 castaglia Exp $
  */
 
-#ifndef __IDENT_H
-#define __IDENT_H
+#ifndef PR_IDENT_H
+#define PR_IDENT_H
 
-char *get_ident(pool *, conn_t *);
+char *pr_ident_lookup(pool *, conn_t *);
 
-#endif /* __IDENT_H */
+#endif /* PR_IDENT_H */
