@@ -2,8 +2,8 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -26,11 +26,11 @@
 
 /* BSD socket manipulation tools.
  *
- * $Id: inet.h,v 1.12 2002/08/14 16:17:35 castaglia Exp $
+ * $Id: inet.h,v 1.17 2003/01/14 05:22:22 castaglia Exp $
  */
 
-#ifndef __INET_H
-#define __INET_H
+#ifndef PR_INET_H
+#define PR_INET_H
 
 /* connection modes */
 #define CM_NONE		0
@@ -55,6 +55,7 @@
 
 /* connection structure */
 typedef struct conn_struc {
+  struct conn_struc *next;
   pool *pool;
   int mode;				/* Current connection mode */
   int listen_fd;			/* Listening file descriptor */
@@ -64,9 +65,9 @@ typedef struct conn_struc {
 
   array_header *iplist;
   int niplist;				/* IPs we are listening to */
-  
+
   int rfd,wfd;				/* Read and write fds */
-  IOFILE *inf,*outf;			/* Input/Output streams */
+  pr_netio_stream_t *instrm, *outstrm;	/* Input/Output streams */
 
   p_in_addr_t *remote_ipaddr;		/* Remote address of connection */
   int remote_port;			/* Remote port of connection */
@@ -97,8 +98,8 @@ void inet_close(pool *, conn_t *);
 void inet_lingering_close(pool *, conn_t *, long);
 int inet_setnonblock(pool *, conn_t *);
 int inet_setblock(pool *, conn_t *);
-int inet_setoptions(pool *, conn_t *, int, int);
-int inet_set_proto_options(pool *, conn_t *, int, int, int, int);
+int inet_set_proto_opts(pool *, conn_t *, int, int, int, int);
+int inet_set_socket_opts(pool *, conn_t *, int, int);
 int inet_setasync(pool *, conn_t *);
 int inet_listen(pool *, conn_t *, int);
 int inet_resetlisten(pool *, conn_t *);
@@ -107,9 +108,9 @@ int inet_connect(pool *, conn_t *, p_in_addr_t *, int);
 int inet_connect_nowait(pool*,conn_t*,p_in_addr_t*,int);
 int inet_get_conn_info(conn_t *, int);
 conn_t *inet_accept(pool *, conn_t *, conn_t *, int, int, unsigned char);
-conn_t *inet_associate(pool *, conn_t *, p_in_addr_t *, IOFILE *,
-  IOFILE *, int);
-conn_t *inet_openrw(pool *, conn_t *, p_in_addr_t *, int, int, int, int);
+conn_t *inet_associate(pool *, conn_t *, p_in_addr_t *,
+  pr_netio_stream_t *, pr_netio_stream_t *, int);
+conn_t *inet_openrw(pool *, conn_t *, p_in_addr_t *, int, int, int, int, int);
 void inet_resolve_ip(pool *, conn_t *);
 
-#endif /* __INET_H */
+#endif /* PR_INET_H */
