@@ -2,8 +2,8 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -26,11 +26,11 @@
 
 /* User configurable defaults and tunable parameters.
  *
- * $Id: options.h,v 1.8 2002/07/02 17:01:47 castaglia Exp $
+ * $Id: options.h,v 1.14 2003/01/02 17:28:17 castaglia Exp $
  */
 
-#ifndef __OPTIONS_H
-#define __OPTIONS_H
+#ifndef PR_OPTIONS_H
+#define PR_OPTIONS_H
 
 /* Define the next option if your libc needs persistant /etc/passwd
  * and /etc/group functions.  Some libcs occasionally close these files
@@ -57,14 +57,19 @@
  * for requests again.
  */
 
-#define TUNABLE_SELECT_TIMEOUT	30
+#define PR_TUNABLE_SELECT_TIMEOUT	30
+
+/* Hash table size is the number of items in the module hash tables.
+ */
+
+#define PR_TUNABLE_HASH_TABLE_SIZE 40
 
 /* "Backlog" is the number of connections that can be received at one
  * burst before the kernel rejects.  This can be configured by the
  * "tcpBackLog" configuration directive, this value is just the default.
  */
 
-#define TUNABLE_DEFAULT_BACKLOG 5
+#define PR_TUNABLE_DEFAULT_BACKLOG 5
 
 /* The next two define the default receive/send tcp windows (and
  * internal ProFTPD buffer sizes.  These can be configured per server
@@ -72,25 +77,43 @@
  * directives.
  */
 
-#define TUNABLE_DEFAULT_RWIN    8192
-#define TUNABLE_DEFAULT_SWIN    8192
+#define PR_TUNABLE_DEFAULT_RWIN    8192
+#define PR_TUNABLE_DEFAULT_SWIN    8192
 
 /* Default internal buffer size used for data transfers and other
  * miscellaneous tasks.
  */
-
-#define TUNABLE_BUFFER_SIZE	1024
+#ifndef PR_TUNABLE_BUFFER_SIZE
+# define PR_TUNABLE_BUFFER_SIZE	1024
+#endif
 
 /* Default timeouts, if not explicitly configured via
  * the TimeoutLogin, TimeoutIdle, etc directives.
  */
 
-#define TUNABLE_TIMEOUTLOGIN	300
-#define TUNABLE_TIMEOUTIDLE	600
-#define TUNABLE_TIMEOUTLINGER	180
-#define TUNABLE_TIMEOUTNOXFER	300
-#define TUNABLE_TIMEOUTIDENT	10
-#define TUNABLE_TIMEOUTSTALLED	3600
+#ifndef PR_TUNABLE_TIMEOUTIDENT
+# define PR_TUNABLE_TIMEOUTIDENT	10
+#endif
+
+#ifndef PR_TUNABLE_TIMEOUTIDLE
+# define PR_TUNABLE_TIMEOUTIDLE	600
+#endif
+
+#ifndef PR_TUNABLE_TIMEOUTLINGER
+# define PR_TUNABLE_TIMEOUTLINGER	180
+#endif
+
+#ifndef PR_TUNABLE_TIMEOUTLOGIN
+# define PR_TUNABLE_TIMEOUTLOGIN	300
+#endif
+
+#ifndef PR_TUNABLE_TIMEOUTNOXFER
+# define PR_TUNABLE_TIMEOUTNOXFER	300
+#endif
+
+#ifndef PR_TUNABLE_TIMEOUTSTALLED
+# define PR_TUNABLE_TIMEOUTSTALLED	3600
+#endif
 
 /* Number of bytes in a new memory pool.  During file transfers,
  * quite a few pools can be created, which eat up a lot of memory.
@@ -98,7 +121,25 @@
  * can negatively impact performance)
  */
 
-#define TUNABLE_NEW_POOL_SIZE   512
+#ifndef PR_TUNABLE_NEW_POOL_SIZE
+# define PR_TUNABLE_NEW_POOL_SIZE	512
+#endif
+
+/* Number of bytes in certain scoreboard fields, usually for reporting
+ * the full command received from the connected client, or the current
+ * working directory for the session.
+ */
+
+#ifndef PR_TUNABLE_SCOREBOARD_BUFFER_SIZE
+# define PR_TUNABLE_SCOREBOARD_BUFFER_SIZE	80
+#endif
+
+/* Max number of recursion/directory levels to support when globbing.
+ */
+
+#ifndef PR_TUNABLE_GLOBBING_MAX	
+# define PR_TUNABLE_GLOBBING_MAX	8
+#endif
 
 /* Loopback network, this should generally not need to be changed,
  * although you can set a specific address by setting it to say
@@ -108,4 +149,4 @@
 #define LOOPBACK_NET            "127.0.0.0"
 #define LOOPBACK_MASK           "255.255.255.0"
 
-#endif /* __OPTIONS_H */
+#endif /* PR_OPTIONS_H */
