@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 /* Memory allocation/anti-leak system.  Yes, this *IS* stolen from Apache
  * also.  What can I say?  It makes sense, and it's safe (more overhead
  * though)
- * $Id: pool.h,v 1.9 2002/07/24 22:20:20 castaglia Exp $
+ * $Id: pool.h,v 1.13 2003/01/02 18:25:18 castaglia Exp $
  */
 
 #ifndef PR_POOL_H
@@ -36,15 +36,16 @@
 typedef struct pool pool;
 
 extern pool *permanent_pool;
-void init_alloc(void);
-pool *make_sub_pool(pool *);		/* All pools are sub-pools of perm */
-pool *make_named_sub_pool(pool *, const char *);
 
-/* Low-level memory allocation */
-void *xmalloc(size_t);
-void *xcalloc(size_t, size_t);
-void *xrealloc(void *, size_t);
-void pool_release_free_block_list(void);
+void pr_init_pools(void);
+pool *make_sub_pool(pool *);		/* All pools are sub-pools of perm */
+
+/* The make_named_sub_pool() function, which was never really supported,
+ * has been deprecated.  This macro is for supporting those contrib modules
+ * that still may make use of this old function.  Newer code should simply
+ * use make_sub_pool() directly.
+ */
+#define make_named_sub_pool(p, s)       make_sub_pool((p))
 
 /* Clears out _everything_ in a pool, destroying any sub-pools */
 void destroy_pool(struct pool *);
@@ -78,12 +79,11 @@ array_header *copy_array(pool *, const array_header *);
 array_header *copy_array_str(pool *, const array_header *);
 array_header *copy_array_hdr(pool *, const array_header *);
 
-/* Alarm signals can easily interfere with the pooled memory operations,
-   thus block_alarms() and unblock_alarms() provide for re-entrant
-   security. */
-
-extern void block_alarms(void);
-extern void unblock_alarms(void);
+/* Alarm signals can easily interfere with the pooled memory operations, thus
+ * pr_alarms_block() and pr_alarms_unblock() provide for re-entrant security.
+ */
+extern void pr_alarms_block(void);
+extern void pr_alarms_unblock(void);
 
 FILE *pfopen(struct pool *, const char *, const char *);
 FILE *pfdopen(struct pool *, int, const char *);
@@ -94,14 +94,11 @@ int pclosef(struct pool *, int);
 
 /* Functions for cleanup handlers */
 void register_cleanup(pool *, void *, void (*)(void *), void (*)(void *));
+void register_file_cleanups(pool *, FILE *);
 void unregister_cleanup(pool *, void *, void (*)(void *));
 
 /* minimum free bytes in a new block pool */
 
-#define BLOCK_MINFREE TUNABLE_NEW_POOL_SIZE
-
-/* accounting */
-long bytes_in_pool(pool *);
-long bytes_in_free_blocks(void);
+#define BLOCK_MINFREE	PR_TUNABLE_NEW_POOL_SIZE
 
 #endif /* PR_POOL_H */
