@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,11 @@
  */
 
 /* Non-specific support functions.
- * $Id: support.h,v 1.11 2002/07/09 22:20:10 castaglia Exp $
+ * $Id: support.h,v 1.18 2003/01/16 02:04:46 castaglia Exp $
  */
 
-#ifndef __SUPPORT_H
-#define __SUPPORT_H
+#ifndef PR_SUPPORT_H
+#define PR_SUPPORT_H
 
 #define CHOP(s)		strip_end((s),"\r\n")
 
@@ -60,17 +60,17 @@ int getopt_long(int, char * const [], const char *, const struct option *,
   int *);
 #endif
 
-void block_signals(void);
-void unblock_signals(void);
+void pr_signals_block(void);
+void pr_signals_unblock(void);
 
 char *dir_interpolate(pool *, const char *);
 char *dir_abs_path(pool *, const char *, int);
 char *dir_realpath(pool *, const char *);
 char *dir_canonical_path(pool *, const char *);
+char *dir_canonical_vpath(pool *, const char *);
 char *dir_best_path(pool *, const char *);
-char *dir_virtual_chdir(pool *, const char *);
 
-void add_exit_handler(void (*f)());
+void pr_exit_register_handler(void (*)(void));
 void remove_exit_handlers(void);
 void run_exit_handlers(void);
 
@@ -92,11 +92,9 @@ char *get_token(char **, char *);
 char *safe_token(char **);
 int check_shutmsg(time_t *, time_t *, time_t *, char *, size_t);
 
+void pr_memscrub(void *, size_t);
 char *sstrcat(char *, const char *, size_t);
 char *sstrncpy(char *, const char *, size_t);
 char *sreplace(pool *, char *, ...);
 
-#if defined(HAVE_SYS_STATVFS_H) || defined(HAVE_SYS_VFS_H)
-off_t get_fs_size(char *);
-#endif
-#endif /* __SUPPORT_H */
+#endif /* PR_SUPPORT_H */
