@@ -22,6 +22,7 @@
 #endif
 
 #include <config.h>
+#include "options.h"
 
 /* Make alloca work the best possible way.  */
 #ifdef __GNUC__
@@ -36,7 +37,7 @@ char *alloca ();
 #endif /* not HAVE_ALLOCA_H */
 #endif /* not __GNUC__ */
 
-#define MAX_RECURSION 8
+#define MAX_RECURSION 	PR_TUNABLE_GLOBBING_MAX
 #define MAX_RESULTS 100000UL
 
 /* Enable GNU extensions in glob.h.  */
@@ -1510,12 +1511,12 @@ glob_in_dir (const char *pattern, const char *directory, int flags,
 
  memory_error:
   {
-    int save = errno;
+    int my_save = errno;
     if (flags & GLOB_ALTDIRFUNC)
       (*pglob->gl_closedir) (stream);
     else
       closedir ((DIR *) stream);
-    __set_errno (save);
+    __set_errno (my_save);
   }
   while (names != NULL)
     {
