@@ -1,8 +1,9 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- *  
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -25,17 +26,16 @@
 
 /*
  * Flexible logging module for proftpd
- * $Id: mod_log.c,v 1.23 2002/05/21 20:47:17 castaglia Exp $
+ * $Id: mod_log.c,v 1.49 2003/02/24 02:37:18 castaglia Exp $
  */
 
 #include "conf.h"
-
 #include "privs.h"
 
-extern response_t *resp_list,*resp_err_list;
+extern pr_response_t *resp_list,*resp_err_list;
 
-#define LOGBUF_SIZE		1025
-#define LOG_EXTENDED_MODE	0644
+#define EXTENDED_LOG_BUFFER_SIZE		1025
+#define EXTENDED_LOG_MODE			0644
 
 typedef struct logformat_struc	logformat_t;
 typedef struct logfile_struc 	logfile_t;
@@ -50,13 +50,23 @@ struct logformat_struc {
 struct logfile_struc {
   logfile_t		*next,*prev;
 
-  int			lf_fd;
-  int			lf_classes;
   char			*lf_filename;
+  int			lf_fd;
+  int			lf_syslog_level;
+
   logformat_t		*lf_format;
-  config_rec		*lf_conf;	/* pointer to the "owning" configuration */
+
+  int			lf_classes;
+
+  /* Pointer to the "owning" configuration */
+  config_rec		*lf_conf;
 };
-  
+
+/* Value for lf_fd signalling that data should be logged via syslog, rather
+ * than written to a file.
+ */
+#define EXTENDED_LOG_SYSLOG	-4
+
 #define META_START		0xff
 #define META_ARG_END		0xfe
 #define META_ARG		1
@@ -66,18 +76,23 @@ struct logfile_struc {
 #define META_REMOTE_HOST	5
 #define META_REMOTE_IP		6
 #define META_IDENT_USER		7
-#define META_SERVER_PORT	8
-#define META_PID		9
-#define META_TIME		10
-#define META_SECONDS		11
-#define META_COMMAND		12
-#define META_SERVERNAME		13
-#define META_USER		14
-#define META_RESPONSE_CODE	15
-#define META_CLASS		16
-#define META_ANON_PASS		17
-#define META_METHOD		18
-#define META_XFER_PATH		19
+#define META_PID		8
+#define META_TIME		9
+#define META_SECONDS		10
+#define META_COMMAND		11
+#define META_LOCAL_NAME		12
+#define META_LOCAL_PORT		13
+#define META_LOCAL_IP		14
+#define META_LOCAL_FQDN		15
+#define META_USER		16
+#define META_ORIGINAL_USER	17
+#define META_RESPONSE_CODE	18
+#define META_CLASS		19
+#define META_ANON_PASS		20
+#define META_METHOD		21
+#define META_XFER_PATH		22
+#define META_DIR_NAME		23
+#define META_DIR_PATH		24
 
 static pool			*log_pool;
 static logformat_t		*formats = NULL;
@@ -86,51 +101,56 @@ static logfile_t		*logs = NULL;
 static xaset_t			*log_set = NULL;
 
 /* format string args:
-   %A			- anonymous username (password given)
-   %c			- class
-   %b			- bytes sent for request
-   %f			- filename
-   %F			- transfer path (filename for client)
-   %{FOOBAR}e		- contents of environment variable FOOBAR
-   %h			- remote host
-   %a			- remote IP-address
-   %l			- remote logname (from identd)
+   %A			- Anonymous username (password given)
+   %a			- Remote client IP address
+   %b			- Bytes sent for request
+   %c			- Class
+   %D			- full directory path
+   %d			- directory (for client)
+   %{FOOBAR}e		- Contents of environment variable FOOBAR
+   %F			- Transfer path (filename for client)
+   %f			- Filename
+   %h			- Remote client DNS name
+   %L                   - Local server IP address
+   %l			- Remote logname (from identd)
    %m			- Request (command) method (RETR, etc.)
-   %p			- Port of server serving request
    %P			- Process ID of child serving request
+   %p			- Port of server serving request
    %r			- Full request (command)
-   %t			- Time
-   %{format}t		- formatted time (strftime(3) format)
+   %s			- Response code (status)
    %T			- Time taken to serve request, in seconds
-   %s			- Reponse code (status)
+   %t			- Time
+   %{format}t		- Formatted time (strftime(3) format)
+   %U                   - Original username sent by client
    %u			- Local user
-   %v			- Servername of server serving request
+   %V                   - DNS name of server serving request
+   %v			- ServerName of server serving request
 */
 
 static void add_meta(unsigned char **s, unsigned char meta, int args,
-		     ...) {
+                     ...) {
   int arglen;
   char *arg;
-  
+
   **s = META_START;
   (*s) = (*s) + 1;
   **s = meta;
   (*s) = (*s) + 1;
-  
-  if(args) {
+
+  if (args) {
     va_list ap;
     va_start(ap, args);
 
     while(args--) {
       arglen = va_arg(ap, int);
       arg = va_arg(ap, char *);
-     
-      memcpy(*s, arg, arglen); 
+
+      memcpy(*s, arg, arglen);
       (*s) = (*s) + arglen;
       **s = META_ARG_END;
       (*s) = (*s) + 1;
     }
-    
+
     va_end(ap);
   }
 }
@@ -160,82 +180,121 @@ void logformat(char *nickname, char *fmts)
    * might overrun the format buffer.  Fixing this problem involves a
    * rewrite of most of this module.  This will happen post 1.2.0.
    */
-  
+
   outs = format;
-  for(tmp = fmts; *tmp; ) {
-    if(*tmp == '%') {
+  for (tmp = fmts; *tmp; ) {
+    if (*tmp == '%') {
       arg = NULL;
       tmp++;
-      for(;;) {
-        switch(*tmp) {
+      for (;;) {
+        switch (*tmp) {
         case '{':
           arg = preparse_arg(&tmp);
           continue;
+
+        case 'a':
+          add_meta(&outs, META_REMOTE_IP, 0);
+          break;
+
         case 'A':
           add_meta(&outs, META_ANON_PASS, 0);
           break;
+
         case 'b':
           add_meta(&outs, META_BYTES_SENT, 0);
           break;
+
         case 'c':
           add_meta(&outs, META_CLASS, 0);
           break;
-        case 'f':
-          add_meta(&outs, META_FILENAME, 0);
+
+        case 'D':
+          add_meta(&outs, META_DIR_PATH, 0);
           break;
-        case 'F':
-          add_meta(&outs, META_XFER_PATH, 0);
+
+        case 'd':
+          add_meta(&outs, META_DIR_NAME, 0);
           break;
+
         case 'e':
-          if(arg) {
+          if (arg) {
             add_meta(&outs, META_ENV_VAR, 0);
             add_meta(&outs, META_ARG, 1, (int) strlen(arg), arg);
           }
           break;
+
+        case 'f':
+          add_meta(&outs, META_FILENAME, 0);
+          break;
+
+        case 'F':
+          add_meta(&outs, META_XFER_PATH, 0);
+          break;
+
         case 'h':
           add_meta(&outs, META_REMOTE_HOST, 0);
           break;
-        case 'a':
-          add_meta(&outs, META_REMOTE_IP, 0);
-          break;
+
         case 'l':
           add_meta(&outs, META_IDENT_USER, 0);
           break;
+
+        case 'L':
+          add_meta(&outs, META_LOCAL_IP, 0);
+          break;
+
         case 'm':
           add_meta(&outs, META_METHOD, 0);
           break;
-        case 'p': 
-          add_meta(&outs, META_SERVER_PORT, 0);
+
+        case 'p':
+          add_meta(&outs, META_LOCAL_PORT, 0);
           break;
+
         case 'P':
           add_meta(&outs, META_PID, 0);
           break;
-        case 't':
-          add_meta(&outs, META_TIME, 0);
-          if(arg)
-            add_meta(&outs, META_ARG, 1, (int) strlen(arg), arg);
-          break;
-        case 'T':
-          add_meta(&outs, META_SECONDS, 0);
-          break;
-        case 'u':
-          add_meta(&outs, META_USER, 0);
-          break;
+
         case 'r':
           add_meta(&outs, META_COMMAND, 0);
           break;
-        case 'v':
-          add_meta(&outs, META_SERVERNAME, 0);
-          break;
+
         case 's':
           add_meta(&outs, META_RESPONSE_CODE, 0);
           break;
+
+        case 't':
+          add_meta(&outs, META_TIME, 0);
+          if (arg)
+            add_meta(&outs, META_ARG, 1, (int) strlen(arg), arg);
+          break;
+
+        case 'T':
+          add_meta(&outs, META_SECONDS, 0);
+          break;
+
+        case 'u':
+          add_meta(&outs, META_USER, 0);
+          break;
+
+        case 'U':
+          add_meta(&outs, META_ORIGINAL_USER, 0);
+          break;
+
+        case 'v':
+          add_meta(&outs, META_LOCAL_NAME, 0);
+          break;
+
+        case 'V':
+          add_meta(&outs, META_LOCAL_FQDN, 0);
+          break;
+
         case '%':
           *outs++ = '%';
           break;
         }
-	tmp++;
-	break;
+        tmp++;
+        break;
       }
     } else {
       *outs++ = *tmp++;
@@ -248,19 +307,16 @@ void logformat(char *nickname, char *fmts)
   lf->lf_nickname = pstrdup(log_pool, nickname);
   lf->lf_format = palloc(log_pool, outs - format);
   memcpy(lf->lf_format, format, outs - format);
-  
-  if(!format_set)
+
+  if (!format_set)
     format_set = xaset_create(log_pool, NULL);
 
   xaset_insert_end(format_set, (xasetmember_t *) lf);
   formats = (logformat_t *) format_set->xas_list;
 }
 
-/* Syntax: LogFormat nickname "format string"
- */
-
-MODRET add_logformat(cmd_rec *cmd)
-{
+/* Syntax: LogFormat nickname "format string" */
+MODRET set_logformat(cmd_rec *cmd) {
   CHECK_ARGS(cmd, 2);
   CHECK_CONF(cmd, CONF_ROOT);
 
@@ -275,75 +331,82 @@ static int _parse_classes(char *s)
 
   do {
 
-    if((nextp = strchr(s, ',')))
+    if ((nextp = strchr(s, ',')))
       *nextp++ = '\0';
 
-    if(!nextp) {
-      if((nextp = strchr(s, '|')))
+    if (!nextp) {
+      if ((nextp = strchr(s, '|')))
         *nextp++ = '\0';
     }
 
-    if(!strcasecmp(s,"NONE"))
+    if (!strcasecmp(s,"NONE"))
       { classes = CL_NONE; break; }
-    if(!strcasecmp(s,"ALL"))
+    if (!strcasecmp(s,"ALL"))
       { classes = CL_ALL; break; }
-    else if(!strcasecmp(s,"AUTH"))
+    else if (!strcasecmp(s,"AUTH"))
       classes |= CL_AUTH;
-    else if(!strcasecmp(s,"INFO"))
+    else if (!strcasecmp(s,"INFO"))
       classes |= CL_INFO;
-    else if(!strcasecmp(s,"DIRS"))
+    else if (!strcasecmp(s,"DIRS"))
       classes |= CL_DIRS;
-    else if(!strcasecmp(s,"READ"))
+    else if (!strcasecmp(s,"READ"))
       classes |= CL_READ;
-    else if(!strcasecmp(s,"WRITE"))
+    else if (!strcasecmp(s,"WRITE"))
       classes |= CL_WRITE;
-    else if(!strcasecmp(s,"MISC"))
+    else if (!strcasecmp(s,"MISC"))
       classes |= CL_MISC;
     else
-      log_pri(LOG_NOTICE, "ExtendedLog class '%s' is not defined.", s);
+      log_pri(PR_LOG_NOTICE, "ExtendedLog class '%s' is not defined.", s);
   } while((s = nextp));
 
   return classes;
 }
 
-/* Syntax: ExtendedLog <log-filename> [<cmd-classes> [<format-nickname>]]
- */
-
-MODRET add_extendedlog(cmd_rec *cmd)
-{
-  config_rec *c;
+/* Syntax: ExtendedLog file [<cmd-classes> [<nickname>]] */
+MODRET set_extendedlog(cmd_rec *cmd) {
+  config_rec *c = NULL;
   int argc;
-  char **argv;
 
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
 
   argc = cmd->argc;
-  argv = cmd->argv;
 
-  if(argc < 2)
-    CONF_ERROR(cmd, "Syntax: ExtendedLog <log-filename> "
-	       "[<Command-Classes> [<Format-Nickname>]]");
+  if (argc < 2)
+    CONF_ERROR(cmd, "Syntax: ExtendedLog file [<cmd-classes> [<nickname>]]");
 
-  c = add_config_param("ExtendedLog",3,NULL,NULL,NULL);
+  c = add_config_param(cmd->argv[0], 3, NULL, NULL, NULL);
 
-  if(cmd->argv[1][0] != '/')
-    c->argv[0] = dir_canonical_path(log_pool,cmd->argv[1]);
-  else
-    c->argv[0] = pstrdup(log_pool,cmd->argv[1]);
+  if (strncasecmp(cmd->argv[1], "syslog:", 7) == 0) {
+    char *tmp = strchr(cmd->argv[1], ':');
 
-  if(argc > 2)
-    c->argv[1] = pstrdup(log_pool,cmd->argv[2]);
-  if(argc > 3)
-    c->argv[2] = pstrdup(log_pool,cmd->argv[3]);
+    if (log_str2sysloglevel(++tmp) < 0) {
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "unknown syslog level: '",
+        tmp, "'", NULL));
+
+    } else
+      c->argv[0] = pstrdup(log_pool, cmd->argv[1]);
+
+  } else if (cmd->argv[1][0] != '/') {
+    CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "relative paths not allowed: '",
+        cmd->argv[1], "'", NULL));
+
+  } else
+    c->argv[0] = pstrdup(log_pool, cmd->argv[1]);
+
+  if (argc > 2)
+    c->argv[1] = pstrdup(log_pool, cmd->argv[2]);
+
+  if (argc > 3)
+    c->argv[2] = pstrdup(log_pool, cmd->argv[3]);
 
   c->argc = argc-1;
   return HANDLED(cmd);
 }
 
 /* Syntax: AllowLogSymlinks <on|off> */
-
 MODRET set_allowlogsymlinks(cmd_rec *cmd) {
-  int bool;
+  int bool = -1;
+  config_rec *c = NULL;
 
   CHECK_ARGS(cmd, 1);
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
@@ -351,20 +414,30 @@ MODRET set_allowlogsymlinks(cmd_rec *cmd) {
   if ((bool = get_boolean(cmd, 1)) == -1)
     CONF_ERROR(cmd, "expected boolean argument.");
 
-  add_config_param("AllowLogSymlinks", 1, (void *) bool);
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
+  *((unsigned char *) c->argv[0]) = bool;
+
+  return HANDLED(cmd);
+}
+
+/* Syntax: ServerLog <filename> */
+MODRET set_serverlog(cmd_rec *cmd) {
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
+
+  add_config_param_str(cmd->argv[0], 1, cmd->argv[1]);
 
   return HANDLED(cmd);
 }
 
 /* Syntax: SystemLog <filename> */
-
-MODRET set_systemlog(cmd_rec *cmd)
-{
-  char *syslogfn;
+MODRET set_systemlog(cmd_rec *cmd) {
+  char *syslogfn = NULL;
   int ret;
-  
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT);
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT);
 
   log_closesyslog();
 
@@ -375,18 +448,18 @@ MODRET set_systemlog(cmd_rec *cmd)
     return HANDLED(cmd);
   }
 
-  if (*syslogfn != '/') 
+  if (*syslogfn != '/')
     syslogfn = dir_canonical_path(cmd->tmp_pool,syslogfn);
 
-  block_signals();
+  pr_signals_block();
   PRIVS_ROOT
 
   if ((ret = log_opensyslog(syslogfn)) < 0) {
     int xerrno = errno;
-      
+
     PRIVS_RELINQUISH
-    unblock_signals();
-    
+    pr_signals_unblock();
+
     if (ret == LOG_WRITEABLE_DIR) {
       CONF_ERROR(cmd,
         "you are attempting to log to a world writeable directory");
@@ -400,9 +473,9 @@ MODRET set_systemlog(cmd_rec *cmd)
         strerror(xerrno), NULL));
     }
   }
-  
+
   PRIVS_RELINQUISH
-  unblock_signals();
+  pr_signals_unblock();
 
   return HANDLED(cmd);
 }
@@ -439,19 +512,17 @@ struct tm *_get_gmtoff(int *tz)
 }
 #endif /* HAVE_GMTOFF */
 
-static
-char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
-{
+static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
   unsigned char *m;
   char arg[512] = {'\0'}, *argp = NULL, *pass;
-  
+
   /* This function can cause potential problems.  Custom logformats
    * might overrun the arg buffer.  Fixing this problem involves a
    * rewrite of most of this module.  This will happen post 1.2.0.
    */
-  
+
   m = (*f) + 1;
-  switch(*m) {
+  switch (*m) {
   case META_ARG:
     m++; argp = arg;
     while(*m != META_ARG_END)
@@ -465,18 +536,18 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
     argp = arg;
 
     pass = get_param_ptr(cmd->server->conf, C_PASS, FALSE);
-    if(!pass)
+    if (!pass)
       pass = "UNKNOWN";
-    
+
     sstrncpy(argp, pass, sizeof(arg));
 
     m++;
     break;
-    
+
   case META_BYTES_SENT:
     argp = arg;
-    if(session.xfer.p)
-      snprintf(argp, sizeof(arg), "%lu",session.xfer.total_bytes);
+    if (session.xfer.p)
+      snprintf(argp, sizeof(arg), "%" PR_LU, session.xfer.total_bytes);
     else
       sstrncpy(argp, "-", sizeof(arg));
 
@@ -484,22 +555,94 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
     break;
 
   case META_CLASS:
+    {
+      unsigned char *classes = get_param_ptr(TOPLEVEL_CONF, "Classes", FALSE);
+      argp = arg;
+
+      if (classes && *classes == TRUE)
+        sstrncpy(argp, session.class->name, sizeof(arg));
+      else
+        sstrncpy(argp, "-", sizeof(arg));
+      m++;
+    }
+    break;
+
+  case META_DIR_NAME:
     argp = arg;
-    if(get_param_int(TOPLEVEL_CONF, "Classes", FALSE) > 0)
-      sstrncpy(argp, session.class->name, sizeof(arg));
-    else
-      sstrncpy(argp, "-", sizeof(arg));
+
+    if (!strcmp(cmd->argv[0], C_CDUP) || !strcmp(cmd->argv[0], C_CWD) ||
+        !strcmp(cmd->argv[0], C_MKD) || !strcmp(cmd->argv[0], C_RMD) ||
+        !strcmp(cmd->argv[0], C_XCWD) || !strcmp(cmd->argv[0], C_XCUP) ||
+        !strcmp(cmd->argv[0], C_XMKD) || !strcmp(cmd->argv[0], C_XRMD)) {
+      char *tmp = strrchr(cmd->arg, '/');
+
+      if (tmp)
+        sstrncpy(argp, tmp, sizeof(arg));
+      else
+        sstrncpy(argp, cmd->arg, sizeof(arg));
+
+    } else {
+      sstrncpy(argp, "", sizeof(arg));
+    }
+
+    m++;
+    break;
+
+  case META_DIR_PATH:
+    argp = arg;
+
+    if (!strcmp(cmd->argv[0], C_CDUP) || !strcmp(cmd->argv[0], C_MKD) ||
+        !strcmp(cmd->argv[0], C_RMD) || !strcmp(cmd->argv[0], C_XCUP) ||
+        !strcmp(cmd->argv[0], C_XMKD) || !strcmp(cmd->argv[0], C_XRMD)) {
+      sstrncpy(argp, dir_abs_path(p, cmd->arg, TRUE), sizeof(arg));
+
+    } else if (!strcmp(cmd->argv[0], C_CWD) || !strcmp(cmd->argv[0], C_XCWD)) {
+
+      /* Note: by this point in the dispatch cycle, the current working
+       * directory has already been changed.  For the CWD/XCWD commands,
+       * this means that dir_abs_path() may return an improper path,
+       * with the target directory being reported twice.  To deal with this,
+       * don't use dir_abs_path(), and use pr_fs_getvwd()/pr_fs_getcwd()
+       * instead.
+       */
+
+      if (session.chroot_path) { 
+        /* Chrooted session. */
+        sstrncpy(arg, strcmp(pr_fs_getvwd(), "/") ?
+          pdircat(p, session.chroot_path, pr_fs_getvwd(), NULL) :
+          session.chroot_path, sizeof(arg));
+
+      } else
+
+        /* Non-chrooted session. */
+        sstrncpy(arg, pr_fs_getcwd(), sizeof(arg));
+
+    } else
+      sstrncpy(argp, "", sizeof(arg));
+
     m++;
     break;
 
   case META_FILENAME:
     argp = arg;
-    if(session.xfer.p && session.xfer.path) {
-      char *fullpath;
-      fullpath = dir_abs_path(p,session.xfer.path,TRUE);
-      sstrncpy(argp, fullpath, sizeof(arg));
+
+    if (session.xfer.p && session.xfer.path) {
+      sstrncpy(argp, dir_abs_path(p, session.xfer.path, TRUE), sizeof(arg));
+
     } else {
-      sstrncpy(argp, "-", sizeof(arg));
+
+      /* Some commands (i.e. DELE, MKD, RMD, XMKD, and XRMD) have associated
+       * filenames that are not stored in the session.xfer structure; these
+       * should be expanded properly as well.
+       */
+      if (!strcmp(cmd->argv[0], C_DELE) || !strcmp(cmd->argv[0], C_MKD) ||
+          !strcmp(cmd->argv[0], C_RMD) || !strcmp(cmd->argv[0], C_XMKD) ||
+          !strcmp(cmd->argv[0], C_XRMD))
+        sstrncpy(arg, dir_abs_path(p, cmd->arg, TRUE), sizeof(arg));
+
+      else
+        /* All other situations get a "-".  */
+        sstrncpy(argp, "-", sizeof(arg));
     }
 
     m++;
@@ -507,10 +650,20 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
 
   case META_XFER_PATH:
     argp = arg;
-    if(session.xfer.p && session.xfer.path) {
+    if (session.xfer.p && session.xfer.path) {
       sstrncpy(argp, session.xfer.path, sizeof(arg));
+
     } else {
-      sstrncpy(argp, "-", sizeof(arg));
+
+      /* Some commands (i.e. DELE) have associated filenames that are not
+       * stored in the session.xfer structure; these should be expanded
+       * properly as well.
+       */
+      if (!strcmp(cmd->argv[0], C_DELE))
+        sstrncpy(arg, cmd->arg, sizeof(arg));
+
+      else
+        sstrncpy(argp, "-", sizeof(arg));
     }
 
     m++;
@@ -520,7 +673,7 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
     argp = arg;
     m++;
 
-    if(*m == META_START && *(m+1) == META_ARG) {
+    if (*m == META_START && *(m+1) == META_ARG) {
       char *env;
 
       env = getenv(get_next_meta(p,cmd,&m));
@@ -553,12 +706,24 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
     m++;
     break;
 
-  case META_SERVER_PORT:
+  case META_LOCAL_PORT:
     argp = arg;
     snprintf(argp, sizeof(arg), "%d", cmd->server->ServerPort);
     m++;
     break;
 
+  case META_LOCAL_IP:
+    argp = arg;
+    sstrncpy(argp, inet_ntoa(*session.c->local_ipaddr), sizeof(arg));
+    m++;
+    break;
+
+  case META_LOCAL_FQDN:
+    argp = arg;
+    sstrncpy(argp, cmd->server->ServerFQDN, sizeof(arg));
+    m++;
+    break;
+
   case META_PID:
     argp = arg;
     snprintf(argp, sizeof(arg), "%u",(unsigned int)getpid());
@@ -575,17 +740,17 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
 
       argp = arg; m++;
 
-      if(*m == META_START && *(m+1) == META_ARG) {
+      if (*m == META_START && *(m+1) == META_ARG) {
         time_fmt = get_next_meta(p,cmd,&m);
         internal_fmt = 0;
       }
       t = *_get_gmtoff(&timz);
       sign = (timz < 0 ? '-' : '+');
-      if(timz < 0)
+      if (timz < 0)
         timz = -timz;
 
       strftime(argp,80,time_fmt,&t);
-      if(internal_fmt)
+      if (internal_fmt)
         snprintf(argp + strlen(argp), sizeof(arg) - strlen(argp),
                 "%c%.2d%.2d]", sign, timz/60, timz%60);
 
@@ -594,12 +759,12 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
 
   case META_SECONDS:
     argp = arg;
-    if(session.xfer.p) {
+    if (session.xfer.p) {
       struct timeval end_time;
 
       gettimeofday(&end_time,NULL);
       end_time.tv_sec -= session.xfer.start_time.tv_sec;
-      if(end_time.tv_usec >= session.xfer.start_time.tv_usec)
+      if (end_time.tv_usec >= session.xfer.start_time.tv_usec)
         end_time.tv_usec -= session.xfer.start_time.tv_usec;
       else {
         end_time.tv_usec = 1000000L - (session.xfer.start_time.tv_usec -
@@ -607,8 +772,8 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
         end_time.tv_sec--;
       }
 
-      snprintf(argp, sizeof(arg), "%lu.%03lu", (unsigned long) end_time.tv_sec,
-	       (unsigned long) (end_time.tv_usec / 1000));
+      snprintf(argp, sizeof(arg), "%ld.%03ld", (time_t) end_time.tv_sec,
+               (time_t) (end_time.tv_usec / 1000));
     } else {
       sstrncpy(argp,"-",sizeof(arg));
     }
@@ -619,30 +784,30 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
   case META_COMMAND:
     argp = arg;
 
-    if(!strcasecmp(cmd->argv[0],"PASS") && session.hide_password)
+    if (!strcasecmp(cmd->argv[0], C_PASS) && session.hide_password)
       sstrncpy(argp, "PASS (hidden)", sizeof(arg));
     else
       sstrncpy(argp, get_full_cmd(cmd), sizeof(arg));
     m++;
     break;
 
-  case META_SERVERNAME:
+  case META_LOCAL_NAME:
     argp = arg;
 
-    sstrncpy(argp,cmd->server->ServerName,sizeof(arg));
+    sstrncpy(argp, cmd->server->ServerName, sizeof(arg));
     m++;
     break;
 
   case META_USER:
     argp = arg;
 
-    if(!session.user) {
+    if (!session.user) {
       char *u;
 
       u = get_param_ptr(cmd->server->conf,"UserName",FALSE);
-      if(!u)
+      if (!u)
         u = "root";
-    
+
       sstrncpy(argp, u, sizeof(arg));
     } else {
       sstrncpy(argp, session.user, sizeof(arg));
@@ -651,21 +816,33 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
     m++;
     break;
 
+  case META_ORIGINAL_USER:
+    {
+      char *login_user = get_param_ptr(main_server->conf, C_USER, FALSE);
+      argp = arg;
+
+      if (login_user)
+        sstrncpy(argp, login_user, sizeof(arg));
+      else
+        sstrncpy(argp, "(none)", sizeof(arg));
+
+      m++;
+      break;
+    }
+
   case META_RESPONSE_CODE:
     {
-      response_t *r;
+      pr_response_t *r;
 
       argp = arg;
       r = (resp_list ? resp_list : resp_err_list);
 
-      for(; r && !r->num; r=r->next) ;
-      if(r && r->num)
+      for (; r && !r->num; r=r->next) ;
+      if (r && r->num)
         sstrncpy(argp,r->num,sizeof(arg));
 
-      /* hack to add return code for proper logging of QUIT command
-       * -tj 2001-10-03
-       */
-      else if (!strcasecmp(cmd->argv[0], "QUIT"))
+      /* Hack to add return code for proper logging of QUIT command. */
+      else if (!strcasecmp(cmd->argv[0], C_QUIT))
         sstrncpy(argp, R_221, sizeof(arg));
 
       else
@@ -677,41 +854,44 @@ char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f)
   }
 
   *f = m;
-  if(argp)
+  if (argp)
     return pstrdup(p, argp);
   else
     return NULL;
 }
 
-static
-void do_log(cmd_rec *cmd, logfile_t *lf)
-{
-  unsigned char *f;
-  size_t size = LOGBUF_SIZE-2;
-  char logbuf[LOGBUF_SIZE] = {'\0'};
-  logformat_t *fmt;
-  char *s,*bp;
+/* from src/log.c */
+extern int syslog_sockfd;
+
+static void do_log(cmd_rec *cmd, logfile_t *lf) {
+  unsigned char *f = NULL;
+  size_t size = EXTENDED_LOG_BUFFER_SIZE-2;
+  char logbuf[EXTENDED_LOG_BUFFER_SIZE] = {'\0'};
+  logformat_t *fmt = NULL;
+  char *s, *bp;
 
   fmt = lf->lf_format;
   f = fmt->lf_format;
   bp = logbuf;
 
-  while(*f && size) {
-    if(*f == META_START) {
-      s = get_next_meta(cmd->tmp_pool,cmd,&f);
-      if(s) {
+  while (*f && size) {
+    if (*f == META_START) {
+      s = get_next_meta(cmd->tmp_pool, cmd, &f);
+
+      if (s) {
         size_t tmp;
 
         tmp = strlen(s);
-        if(tmp > size)
+        if (tmp > size)
           tmp = size;
 
         memcpy(bp, s, tmp);
         size -= tmp;
         bp += tmp;
       }
+
     } else {
-      *bp++ = (char)*f++;
+      *bp++ = (char) *f++;
       size--;
     }
   }
@@ -719,31 +899,34 @@ void do_log(cmd_rec *cmd, logfile_t *lf)
   *bp++ = '\n';
   *bp = '\0';
 
-  write(lf->lf_fd,logbuf,strlen(logbuf));
+  if (lf->lf_fd != EXTENDED_LOG_SYSLOG)
+    write(lf->lf_fd, logbuf, strlen(logbuf));
+
+  else
+    pr_syslog(syslog_sockfd, lf->lf_syslog_level, "%s", logbuf);
 }
 
-MODRET log_command(cmd_rec *cmd)
-{
-  logfile_t *lf;
-  /* If not in anon mode, only handle logs for main servers */
+MODRET log_any(cmd_rec *cmd) {
+  logfile_t *lf = NULL;
 
-  for(lf = logs; lf; lf=lf->next)
-    if(lf->lf_fd != -1 && (cmd->class & lf->lf_classes)) {
-      if(!session.anon_config && lf->lf_conf && lf->lf_conf->config_type == CONF_ANON)
+  /* If not in anon mode, only handle logs for main servers */
+  for (lf = logs; lf; lf = lf->next)
+    if (lf->lf_fd != -1 && (cmd->class & lf->lf_classes)) {
+      if (!session.anon_config && lf->lf_conf &&
+          lf->lf_conf->config_type == CONF_ANON)
         continue;
-      do_log(cmd,lf);
+
+      do_log(cmd, lf);
     }
 
   return DECLINED(cmd);
 }
 
-/* log_rehash is called whenever the master server rehashes it's
+/* log_rehash_cb() is called whenever the master server rehashes it's
  * config (in response to SIGHUP).
  */
 
-static
-void log_rehash(void *d)
-{
+static void log_rehash_cb(void *d) {
   destroy_pool(log_pool);
 
   formats = NULL;
@@ -752,20 +935,20 @@ void log_rehash(void *d)
   log_set = NULL;
 
   log_pool = make_sub_pool(permanent_pool);
-  logformat("","%h %l %u %t \"%r\" %s %b");
+  logformat("", "%h %l %u %t \"%r\" %s %b");
 }
-  
-static 
-int log_init(void)
-{
+
+static int log_init(void) {
   log_pool = make_sub_pool(permanent_pool);
-  /* add the "default" extendedlog format */
-  logformat("","%h %l %u %t \"%r\" %s %b");
-  register_rehash(NULL,log_rehash);
+
+  /* Add the "default" extendedlog format */
+  logformat("", "%h %l %u %t \"%r\" %s %b");
+
+  pr_rehash_register_handler(NULL, log_rehash_cb);
   return 0;
 }
 
-static void get_extendedlogs(void) {
+static void find_extendedlogs(void) {
   config_rec *c;
   char *logfname;
   int logclasses = CL_ALL;
@@ -773,44 +956,55 @@ static void get_extendedlogs(void) {
   char *logfmt_s = NULL;
   logfile_t *logf;
 
-  c = find_config(main_server->conf,CONF_PARAM,"ExtendedLog",TRUE);
+  /* We _do_ actually want the recursion here.  The reason is that we want
+   * to find _all_ ExtendedLog directives in the configuration, including
+   * those in <Anonymous> sections.  We have the ability to use root privs
+   * now, to make sure these files can be opened, but after the user has
+   * authenticated (and we know for sure whether they're anonymous or not),
+   * root privs may be permanently revoked.  Yucky...but necessary, I guess.
+   */
 
-  while(c) {
+  c = find_config(main_server->conf, CONF_PARAM, "ExtendedLog", TRUE);
+
+  while (c) {
     logfname = c->argv[0];
 
-    if(c->argc > 1) {
+    if (c->argc > 1) {
       logclasses = _parse_classes(c->argv[1]);
-      if(c->argc > 2)
+      if (c->argc > 2)
         logfmt_s = c->argv[2];
     }
-    
+
     /* No logging for this round.
      */
-    if(logclasses == CL_NONE)
+    if (logclasses == CL_NONE)
       goto loop_extendedlogs;
-    
-    if(logfmt_s) {
+
+    if (logfmt_s) {
       /* search for the format-nickname */
-      for(logfmt = formats; logfmt; logfmt=logfmt->next)
-        if(!strcmp(logfmt->lf_nickname,logfmt_s))
+      for (logfmt = formats; logfmt; logfmt = logfmt->next)
+        if (!strcmp(logfmt->lf_nickname, logfmt_s))
           break;
 
-      if(!logfmt) {
-        log_pri(LOG_NOTICE, "Format-Nickname '%s' is not defined.",
-                           logfmt_s);
+      if (!logfmt) {
+        log_pri(PR_LOG_NOTICE, "ExtendedLog '%s' uses unknown format nickname "
+          "'%s'", logfname, logfmt_s);
         goto loop_extendedlogs;
       }
+
     } else {
       logfmt = formats;
     }
-    
+
     logf = (logfile_t *) pcalloc(permanent_pool, sizeof(logfile_t));
-    logf->lf_fd = -1;
-    logf->lf_classes = logclasses;
+
     logf->lf_filename = pstrdup(permanent_pool, logfname);
+    logf->lf_fd = -1;
+    logf->lf_syslog_level = -1;
+    logf->lf_classes = logclasses;
     logf->lf_format = logfmt;
     logf->lf_conf = c->parent;
-    if(!log_set)
+    if (!log_set)
       log_set = xaset_create(permanent_pool, NULL);
 
     xaset_insert(log_set, (xasetmember_t *) logf);
@@ -821,93 +1015,126 @@ loop_extendedlogs:
   }
 }
 
-MODRET log_auth_complete(cmd_rec *cmd)
-{
+MODRET log_post_pass(cmd_rec *cmd) {
   logfile_t *lf;
 
-  /* authentication is complete, if we aren't in anon-mode, close
+  /* Authentication is complete, if we aren't in anon-mode, close
    * all extendedlogs opened inside <Anonymous> blocks.
    */
-  if(!session.anon_config) {
-    for(lf = logs; lf; lf=lf->next)
-      if(lf->lf_fd != -1 && lf->lf_conf && lf->lf_conf->config_type == CONF_ANON) {
+  if (!session.anon_config) {
+    for (lf = logs; lf; lf = lf->next) {
+      if (lf->lf_fd != -1 && lf->lf_fd != EXTENDED_LOG_SYSLOG &&
+          lf->lf_conf && lf->lf_conf->config_type == CONF_ANON) {
+        log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'", lf->lf_filename);
         close(lf->lf_fd);
         lf->lf_fd = -1;
       }
+    }
+
   } else {
-    /* close all logs which were opened inside a _different_ anonymous
+    /* Close all logs which were opened inside a _different_ anonymous
      * context.
      */
-    for(lf = logs; lf; lf=lf->next)
-      if(lf->lf_fd != -1 && lf->lf_conf && lf->lf_conf != session.anon_config) {
+    for (lf = logs; lf; lf = lf->next) {
+      if (lf->lf_fd != -1 && lf->lf_fd != EXTENDED_LOG_SYSLOG &&
+          lf->lf_conf && lf->lf_conf != session.anon_config) {
+        log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'", lf->lf_filename);
         close(lf->lf_fd);
         lf->lf_fd = -1;
       }
+    }
 
-    /* if any extendedlogs set inside our context match an outer log,
+    /* If any ExtendedLogs set inside our context match an outer log,
      * close the outer (this allows overriding inside <Anonymous>).
      */
-    for(lf = logs; lf; lf=lf->next)
-      if(lf->lf_conf && lf->lf_conf == session.anon_config) {
-        /* this should "override" any lower-level extendedlog with the
+    for (lf = logs; lf; lf = lf->next) {
+      if (lf->lf_conf && lf->lf_conf == session.anon_config) {
+        /* This should "override" any lower-level extendedlog with the
          * same filename.
          */
-        logfile_t *lf2;
+        logfile_t *lfi = NULL;
 
-        for(lf2 = logs; lf2; lf2=lf2->next) {
-          if(lf2->lf_fd != -1 && !lf2->lf_conf &&
-             !strcmp(lf2->lf_filename,lf->lf_filename)) {
-            close(lf2->lf_fd);
-            lf2->lf_fd = -1;
+        for (lfi = logs; lfi; lfi = lfi->next) {
+          if (lfi->lf_fd != -1 && lfi->lf_fd != EXTENDED_LOG_SYSLOG &&
+              !lfi->lf_conf && !strcmp(lfi->lf_filename, lf->lf_filename)) {
+            log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
+              lf->lf_filename);
+            close(lfi->lf_fd);
+            lfi->lf_fd = -1;
           }
         }
-       
-        /* go ahead and close the log if it's CL_NONE */
 
-        if(lf->lf_fd != -1 && lf->lf_classes == CL_NONE) {
+        /* Go ahead and close the log if it's CL_NONE */
+        if (lf->lf_fd != -1 && lf->lf_fd != EXTENDED_LOG_SYSLOG &&
+            lf->lf_classes == CL_NONE) {
+          log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
+            lf->lf_filename);
           close(lf->lf_fd);
           lf->lf_fd = -1;
         }
       }
+    }
   }
+
   return DECLINED(cmd);
 }
 
-static
-int log_child_init(void)
-{
-  /* open all log files */
+/* Open all the log files */
+static int log_sess_init(void) {
+  char *serverlog_name = NULL;
   logfile_t *lf = NULL;
 
-  get_extendedlogs();
+  /* Open the ServerLog, if present. */
+  if ((serverlog_name = get_param_ptr(main_server->conf, "ServerLog",
+      FALSE)) != NULL) {
+    PRIVS_ROOT
+    log_closesyslog();
+    log_opensyslog(serverlog_name);
+    PRIVS_RELINQUISH
+  }
+
+  /* Open all the ExtendedLog files. */
+  find_extendedlogs();
 
   for (lf = logs; lf; lf = lf->next) {
 
     if (lf->lf_fd == -1) {
-      int res = 0;
 
-      block_signals();
-      PRIVS_ROOT
-      res = log_openfile(lf->lf_filename, &lf->lf_fd, LOG_EXTENDED_MODE);
-      PRIVS_RELINQUISH
-      unblock_signals();
+      /* Is this ExtendedLog to be written to a file, or to syslog? */
+      if (strncasecmp(lf->lf_filename, "syslog:", 7) != 0) {
+        int res = 0;
 
-      if (res == -1) {
-        log_pri(LOG_NOTICE, "unable to open ExtendedLog '%s': %s",
-          lf->lf_filename, strerror(errno));
-        continue;
+        log_debug(DEBUG7, "mod_log: opening ExtendedLog '%s'", lf->lf_filename);
 
-      } else if (res == LOG_WRITEABLE_DIR) {
-        log_pri(LOG_NOTICE, "unable to open ExtendedLog '%s': "
-          "containing directory is world writeable", lf->lf_filename);
-        continue;
+        pr_signals_block();
+        PRIVS_ROOT
+        res = log_openfile(lf->lf_filename, &lf->lf_fd, EXTENDED_LOG_MODE);
+        PRIVS_RELINQUISH
+        pr_signals_unblock();
 
-      } else if (res == LOG_SYMLINK) {
-        log_pri(LOG_NOTICE, "unable to open ExtendedLog '%s': "
-          "%s is a symbolic link", lf->lf_filename, lf->lf_filename);
-        close(lf->lf_fd);
-        lf->lf_fd = -1;
-        continue;
+        if (res == -1) {
+          log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': %s",
+            lf->lf_filename, strerror(errno));
+          continue;
+
+        } else if (res == LOG_WRITEABLE_DIR) {
+          log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': "
+            "containing directory is world writeable", lf->lf_filename);
+          continue;
+
+        } else if (res == LOG_SYMLINK) {
+          log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': "
+            "%s is a symbolic link", lf->lf_filename, lf->lf_filename);
+          close(lf->lf_fd);
+          lf->lf_fd = -1;
+          continue;
+        }
+
+      } else {
+        char *tmp = strchr(lf->lf_filename, ':');
+
+        lf->lf_syslog_level = log_str2sysloglevel(++tmp);
+        lf->lf_fd = EXTENDED_LOG_SYSLOG;
       }
     }
   }
@@ -915,29 +1142,47 @@ int log_child_init(void)
   return 0;
 }
 
+/* Module API tables
+ */
 
-static conftable log_config[] = {
+static conftable log_conftab[] = {
   { "AllowLogSymlinks",	set_allowlogsymlinks,			NULL },
-  { "LogFormat",	add_logformat,				NULL },
-  { "ExtendedLog",	add_extendedlog,			NULL },
+  { "ExtendedLog",	set_extendedlog,			NULL },
+  { "LogFormat",	set_logformat,				NULL },
+  { "ServerLog",	set_serverlog,				NULL },
   { "SystemLog",	set_systemlog,				NULL },
   { NULL,		NULL,					NULL }
 };
 
-static cmdtable log_commands[] = {
-  { PRE_CMD, 		C_QUIT,	G_NONE,	log_command,		FALSE, FALSE },
-  { LOG_CMD,		C_ANY,	G_NONE,	log_command,		FALSE, FALSE },
-  { LOG_CMD_ERR,	C_ANY,	G_NONE,	log_command,		FALSE, FALSE },
-  { POST_CMD,		C_PASS,	G_NONE,	log_auth_complete,	FALSE, FALSE },
+static cmdtable log_cmdtab[] = {
+  { PRE_CMD, 		C_QUIT,	G_NONE,	log_any,		FALSE, FALSE },
+  { LOG_CMD,		C_ANY,	G_NONE,	log_any,		FALSE, FALSE },
+  { LOG_CMD_ERR,	C_ANY,	G_NONE,	log_any,		FALSE, FALSE },
+  { POST_CMD,		C_PASS,	G_NONE,	log_post_pass,		FALSE, FALSE },
   { 0, NULL }
 };
 
 module log_module = {
-  NULL,NULL,			/* Always NULL */
-  0x20,				/* API version */
-  "log",			/* Module name */
-  log_config,
-  log_commands,
+  NULL, NULL,
+
+  /* Module API version */
+  0x20,
+
+  /* Module name */
+  "log",
+
+  /* Module configuration handler table */
+  log_conftab,
+
+  /* Module command handler table */
+  log_cmdtab,
+
+  /* Module authentication handler table */
   NULL,
-  log_init,log_child_init
+
+  /* Module initialization */
+  log_init,
+
+  /* Session initialization */
+  log_sess_init
 };
