@@ -1,8 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -30,22 +30,22 @@
 #include "conf.h"
 
 static int ident_timeout;
-static IOFILE *io;
+static pr_netio_stream_t *nstrm = NULL;
 
-static int _ident_timeout(CALLBACK_FRAME)
-{
+static int ident_timeout_cb(CALLBACK_FRAME) {
   ident_timeout++;
 
-  if(io)
-    /* Abort the IOFILE, which will cause io_poll (and thus io_read) to
-     * also abort.  This is similar to the way data connects are aborted
+  if (nstrm)
+    /* Abort the NetIO stream, which will cause netio_poll (and thus
+     * netio_read) to also abort.  This is similar to the way data connects
+     * are aborted.
      */
-    io_abort(io);
-    
+    pr_netio_abort(nstrm);
+
   return 0;
 }
 
-char *get_ident(pool *p, conn_t *c) {
+char *pr_ident_lookup(pool *p, conn_t *c) {
   char *ret = "UNKNOWN";
   pool *tmp_pool = NULL;
   conn_t *ident_conn = NULL, *ident_io = NULL;
@@ -55,20 +55,20 @@ char *get_ident(pool *p, conn_t *c) {
 
   tmp_pool = make_sub_pool(p);
   ident_timeout = 0;
-  io = NULL;
+  nstrm = NULL;
 
   if (ident_port == -1) {
     destroy_pool(tmp_pool);
-    return pstrdup(p, ret);    
+    return pstrdup(p, ret);
   }
- 
+
   /* Set up our timer before going any further. */
-  if ((timer = add_timer(TUNABLE_TIMEOUTIDENT, -1, NULL,
-      (callback_t) _ident_timeout)) <= 0) {
+  if ((timer = add_timer(PR_TUNABLE_TIMEOUTIDENT, -1, NULL,
+      (callback_t) ident_timeout_cb)) <= 0) {
     destroy_pool(tmp_pool);
     return pstrdup(p, ret);
   }
- 
+
   ident_conn = inet_create_connection(tmp_pool, NULL, -1, c->local_ipaddr,
     INPORT_ANY, FALSE);
   inet_setnonblock(tmp_pool, ident_conn);
@@ -80,19 +80,20 @@ char *get_ident(pool *p, conn_t *c) {
     destroy_pool(tmp_pool);
     return pstrdup(p, ret);
   }
- 
-  if (!i) {
-    /* Not yet connected. */ 
-    io = io_open(p, ident_conn->listen_fd, IO_READ);
-    io_set_poll_sleep(io, 1);
 
-    switch (io_poll(io)) {
+  if (!i) {
+    /* Not yet connected. */
+    nstrm = pr_netio_open(p, PR_NETIO_STRM_OTHR, ident_conn->listen_fd,
+      PR_NETIO_IO_RD);
+    pr_netio_set_poll_interval(nstrm, 1);
+
+    switch (pr_netio_poll(nstrm)) {
 
       /* Aborted, timed out */
       case 1:
         if (ident_timeout) {
           remove_timer(timer, NULL);
-          io_close(io);
+          pr_netio_close(nstrm);
           inet_close(tmp_pool, ident_conn);
           destroy_pool(tmp_pool);
           return pstrdup(p, ret);
@@ -102,7 +103,7 @@ char *get_ident(pool *p, conn_t *c) {
       /* Error. */
       case -1:
         remove_timer(timer, NULL);
-        io_close(io);
+        pr_netio_close(nstrm);
         inet_close(tmp_pool, ident_conn);
         destroy_pool(tmp_pool);
         return pstrdup(p, ret);
@@ -113,7 +114,7 @@ char *get_ident(pool *p, conn_t *c) {
 
         if (inet_get_conn_info(ident_conn, ident_conn->listen_fd) < 0) {
           remove_timer(timer, NULL);
-          io_close(io);
+          pr_netio_close(nstrm);
           inet_close(tmp_pool, ident_conn);
           destroy_pool(tmp_pool);
           return pstrdup(p, ret);
@@ -122,42 +123,43 @@ char *get_ident(pool *p, conn_t *c) {
     }
   }
 
-  if ((ident_io = inet_openrw(tmp_pool, ident_conn, NULL, -1, -1, -1,
-      FALSE)) == NULL) {
+  if ((ident_io = inet_openrw(tmp_pool, ident_conn, NULL, PR_NETIO_STRM_OTHR,
+      -1, -1, -1, FALSE)) == NULL) {
     remove_timer(timer, NULL);
     inet_close(tmp_pool, ident_conn);
     destroy_pool(tmp_pool);
     return pstrdup(p, ret);
   }
 
-  io = ident_io->inf;
+  nstrm = ident_io->instrm;
 
   inet_setnonblock(tmp_pool, ident_io);
-  io_set_poll_sleep(ident_io->inf, 1);
-  io_set_poll_sleep(ident_io->outf, 1);
+  pr_netio_set_poll_interval(ident_io->instrm, 1);
+  pr_netio_set_poll_interval(ident_io->outstrm, 1);
 
-  io_printf(ident_io->outf, "%d, %d\r\n", c->remote_port, c->local_port);
+  pr_netio_printf(ident_io->outstrm, "%d, %d\r\n", c->remote_port,
+    c->local_port);
 
-  /* If the timer fires while in io_gets, io_gets will simply return
-   * either a partial string, or NULL.  This works because _ident_timeout
-   * aborts the IOFILE we are reading on.  io_set_poll_sleep() is used
-   * to make sure significant delays don't occur on systems that
+  /* If the timer fires while in netio_gets(), netio_gets() will simply return
+   * either a partial string, or NULL.  This works because ident_timeout_cb
+   * aborts the stream from which we are reading.  netio_set_poll_interval() is
+   * used to make sure significant delays don't occur on systems that
    * automatically restart syscalls after the SIGALRM signal.
    */
-  
-  if (io_gets(buf, sizeof(buf), ident_io->inf)) {
+
+  if (pr_netio_gets(buf, sizeof(buf), ident_io->instrm)) {
     strip_end(buf, "\r\n");
-    
+
     tmp = buf;
     tok = get_token(&tmp, ":");
     if (tok && (tok = get_token(&tmp, ":"))) {
-      while(*tok && isspace((UCHAR)*tok))
+      while (*tok && isspace((int) *tok))
         tok++;
       strip_end(tok, " \t");
 
       if (strcasecmp(tok, "ERROR") == 0) {
-        if(tmp) {
-          while(*tmp && isspace((UCHAR)*tmp))
+        if (tmp) {
+          while (*tmp && isspace((int) *tmp))
             tmp++;
 	  strip_end(tmp, " \t");
           if (strcasecmp(tmp, "HIDDEN-USER") == 0)
@@ -167,7 +169,7 @@ char *get_ident(pool *p, conn_t *c) {
       } else if (strcasecmp(tok, "USERID") == 0) {
         if (tmp && (tok = get_token(&tmp, ":"))) {
           if (tmp) {
-            while (*tmp && isspace((UCHAR)*tmp))
+            while (*tmp && isspace((int) *tmp))
               tmp++;
             strip_end(tmp, " \t");
             ret = tmp;
