@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 /*
  * Resource allocation code
+ * $Id: pool.c,v 1.31 2003/02/12 08:46:36 castaglia Exp $
  */
 
 #include "conf.h"
@@ -34,7 +35,7 @@
 
 union align {
   char *cp;
-  void (*f)();
+  void (*f)(void);
   long l;
   FILE *fp;
   double d;
@@ -66,42 +67,45 @@ static unsigned int stat_freehit = 0;	/* incr when freelist used */
 static void *null_alloc(size_t size) {
   void *ret = 0;
 
-  if(size == 0)
+  if (size == 0)
     ret = malloc(size);
-  if(ret == 0) {
-    log_pri(LOG_ERR,"Fatal: Memory exhausted.");
+  if (ret == 0) {
+    log_pri(PR_LOG_ERR, "fatal: Memory exhausted");
     exit(1);
   }
 
   return ret;
 }
 
-void *xmalloc(size_t size) {
+static void *xmalloc(size_t size) {
   void *ret;
 
   ret = malloc(size);
-  if(ret == 0)
+  if (ret == 0)
     ret = null_alloc(size);
+
   return ret;
 }
 
+#if 0
 void *xcalloc(size_t num, size_t size) {
   void *ret;
 
   ret = calloc(num,size);
-  if(ret == 0)
+  if (ret == 0)
     ret = null_alloc(num * size);
   return ret;
 }
 
 void *xrealloc(void *p, size_t size) {
-  if(p == 0)
+  if (p == 0)
     return xmalloc(size);
   p = realloc(p,size);
-  if(p == 0)
+  if (p == 0)
     p = null_alloc(size);
   return p;
 }
+#endif
 
 /* Grab a completely new block from the system pool.  Relies on malloc()
  * to return truly aligned memory.
@@ -109,10 +113,10 @@ void *xrealloc(void *p, size_t size) {
 
 static union block_hdr *malloc_block(int size) {
   union block_hdr *blok =
-    (union block_hdr*)xmalloc(size + sizeof(union block_hdr));
+    (union block_hdr *) xmalloc(size + sizeof(union block_hdr));
 
   blok->h.next = NULL;
-  blok->h.first_avail = (char *)(blok+1);
+  blok->h.first_avail = (char *) (blok + 1);
   blok->h.endp = size + blok->h.first_avail;
 
   return blok;
@@ -121,9 +125,10 @@ static union block_hdr *malloc_block(int size) {
 static void chk_on_blk_list(union block_hdr *blok, union block_hdr *free_blk) {
   /* Debug code */
 
-  while(free_blk) {
-    if(free_blk == blok) {
-      log_pri(LOG_ERR,"Fatal: DEBUG: Attempt to free already free block in chk_on_blk_list().");
+  while (free_blk) {
+    if (free_blk == blok) {
+      log_pri(PR_LOG_ERR, "Fatal: DEBUG: Attempt to free already free block "
+       "in chk_on_blk_list()");
       exit(1);
     }
 
@@ -140,21 +145,21 @@ static void free_blocks(union block_hdr *blok) {
 
   union block_hdr *old_free_list = block_freelist;
 
-  if(!blok)
+  if (!blok)
     return;		/* Shouldn't be freeing an empty pool */
 
   block_freelist = blok;
 
   /* Adjust first_avail pointers */
 
-  while(blok->h.next) {
-    chk_on_blk_list(blok,old_free_list);
-    blok->h.first_avail = (char *)(blok + 1);
+  while (blok->h.next) {
+    chk_on_blk_list(blok, old_free_list);
+    blok->h.first_avail = (char *) (blok + 1);
     blok = blok->h.next;
   }
 
-  chk_on_blk_list(blok,old_free_list);
-  blok->h.first_avail = (char*)(blok + 1);
+  chk_on_blk_list(blok, old_free_list);
+  blok->h.first_avail = (char *) (blok + 1);
   blok->h.next = old_free_list;
 }
 
@@ -163,39 +168,40 @@ static void free_blocks(union block_hdr *blok) {
  */
 
 static union block_hdr *new_block(int min_size) {
-  int biggest = 0;
   union block_hdr **lastptr = &block_freelist;
   union block_hdr *blok = block_freelist;
 
   min_size = 1 + ((min_size - 1) / BLOCK_MINFREE);
   min_size *= BLOCK_MINFREE;
 
-  while(blok) {
-    biggest = blok->h.endp - blok->h.first_avail;
-    if(min_size <= blok->h.endp - blok->h.first_avail) {
-      /* It's available */
+  /* Check if we have anything of the requested size on our free list first...
+   */
+  while (blok) {
+    if (min_size <= blok->h.endp - blok->h.first_avail) {
       *lastptr = blok->h.next;
-      stat_freehit++;
       blok->h.next = NULL;
+
+      stat_freehit++;
       return blok;
+
     } else {
       lastptr = &blok->h.next;
       blok = blok->h.next;
     }
   }
 
-  /* malloc a new one */
+  /* Nope...damn.  Have to malloc() a new one. */
   stat_malloc++;
   return malloc_block(min_size);
 }
 
-/* accounting */
+/* Accounting */
 
 static unsigned long bytes_in_block_list(union block_hdr *blok) {
   unsigned long size = 0;
 
   while(blok) {
-    size += blok->h.endp - (char*)(blok+1);
+    size += blok->h.endp - (char *) (blok + 1);
     blok = blok->h.next;
   }
 
@@ -217,7 +223,6 @@ struct pool {
   struct pool *sub_prev;
   struct pool *parent;
   char *free_first_avail;
-  char symbol;
 };
 
 pool *permanent_pool = NULL;
@@ -240,38 +245,29 @@ static long __walk_pools(pool *p, int level)
   char _levelpad[80] = "";
   long total = 0;
 
-  if(!p)
+  if (!p)
     return 0;
 
-  if(level > 1) {
+  if (level > 1) {
     memset(_levelpad,' ',sizeof(_levelpad)-1);
-    if((level - 1) * 3 >= sizeof(_levelpad))
+    if ((level - 1) * 3 >= sizeof(_levelpad))
       _levelpad[sizeof(_levelpad)-1] = 0;
     else
       _levelpad[(level - 1) * 3] = '\0';
   }
 
-  for(; p; p = p->sub_next) {
+  for (; p; p = p->sub_next) {
     total += bytes_in_block_list(p->first);
-    if(level == 0) {
-      if(p->symbol)
-        log_pri(LOG_NOTICE,"(%s)0x%08lx bytes", &p->symbol,
-			bytes_in_block_list(p->first));
-      else
-        log_pri(LOG_NOTICE,"0x%08lx bytes",
-	  	      	bytes_in_block_list(p->first));
-    } else {
-      if(p->symbol)
-	log_pri(LOG_NOTICE,"%s(%s)\\- 0x%08lx bytes",_levelpad, &p->symbol,
-			bytes_in_block_list(p->first));
-      else
-        log_pri(LOG_NOTICE,"%s\\- 0x%08lx bytes",_levelpad,
-              bytes_in_block_list(p->first));
-    }
-    
-    /* recurse */
-    if(p->sub_pools)
-      total += __walk_pools(p->sub_pools,level+1);  
+    if (level == 0)
+      log_pri(PR_LOG_NOTICE, "0x%08lx bytes", bytes_in_block_list(p->first));
+
+    else
+      log_pri(PR_LOG_NOTICE, "%s\\- 0x%08lx bytes", _levelpad,
+        bytes_in_block_list(p->first));
+
+    /* Recurse */
+    if (p->sub_pools)
+      total += __walk_pools(p->sub_pools, level+1);
   }
 
   return total;
@@ -279,89 +275,93 @@ static long __walk_pools(pool *p, int level)
 
 static void debug_pool_info(void) {
   if (block_freelist)
-    log_pri(LOG_NOTICE, "Free block list: 0x%08lx bytes",
+    log_pri(PR_LOG_NOTICE, "Free block list: 0x%08lx bytes",
       bytes_in_block_list(block_freelist));
   else
-    log_pri(LOG_NOTICE, "Free block list: EMPTY");
+    log_pri(PR_LOG_NOTICE, "Free block list: EMPTY");
 
-  log_pri(LOG_NOTICE, "%u count blocks malloc'd.", stat_malloc);
-  log_pri(LOG_NOTICE, "%u count blocks reused.", stat_freehit); 
+  log_pri(PR_LOG_NOTICE, "%u count blocks malloc'd.", stat_malloc);
+  log_pri(PR_LOG_NOTICE, "%u count blocks reused.", stat_freehit);
 }
 
 void debug_walk_pools(void) {
-  log_pri(LOG_NOTICE, "Memory pool allocation:");
-  log_pri(LOG_NOTICE, "Total 0x%08lx bytes allocated",
+  log_pri(PR_LOG_NOTICE, "Memory pool allocation:");
+  log_pri(PR_LOG_NOTICE, "Total 0x%08lx bytes allocated",
     __walk_pools(permanent_pool, 0));
   debug_pool_info();
 }
 
+
+#if 0
+/* NOTE: not used at present */
+
 /* Release the entire free block list */
-void pool_release_free_block_list(void)
-{
+static void pool_release_free_block_list(void) {
   union block_hdr *blok,*next;
 
-  block_alarms();
-  
+  pr_alarms_block();
+
   blok = block_freelist;
-  if(blok) {
-    for(next = blok->h.next; next; blok = next, next = blok->h.next)
+  if (blok) {
+    for (next = blok->h.next; next; blok = next, next = blok->h.next)
       free(blok);
   }
   block_freelist = NULL;
 
-  unblock_alarms();
+  pr_alarms_unblock();
 }
+#endif
 
-struct pool *make_named_sub_pool(struct pool *p, const char *symbol)
-{
+struct pool *make_sub_pool(struct pool *p) {
   union block_hdr *blok;
   pool *new_pool;
 
-  block_alarms();
+  pr_alarms_block();
 
   blok = new_block(0);
-  new_pool = (pool *) blok->h.first_avail;
-  
-  blok->h.first_avail += POOL_HDR_BYTES;
-  memset((char *) new_pool, 0, sizeof(struct pool));
 
+  new_pool = (pool *) blok->h.first_avail;
+  blok->h.first_avail += POOL_HDR_BYTES;
+
+  memset(new_pool, 0, sizeof(struct pool));
   new_pool->free_first_avail = blok->h.first_avail;
   new_pool->first = new_pool->last = blok;
-  
-  if(p) {
+
+  if (p) {
     new_pool->parent = p;
     new_pool->sub_next = p->sub_pools;
-    if(new_pool->sub_next)
+
+    if (new_pool->sub_next)
       new_pool->sub_next->sub_prev = new_pool;
+
     p->sub_pools = new_pool;
   }
 
-  unblock_alarms();
+  pr_alarms_unblock();
 
   return new_pool;
 }
 
-struct pool *make_sub_pool(struct pool *p) {
-  return make_named_sub_pool(p, NULL);
-}
-
 /* Initialize the pool system by creating the base permanent_pool. */
 
-void init_alloc(void) {
-  permanent_pool = make_named_sub_pool(NULL, "permanent_pool");
+void pr_init_pools(void) {
+  if (!permanent_pool)
+    permanent_pool = make_sub_pool(NULL);
 }
 
-static void clear_pool(struct pool *p)
-{
+static void clear_pool(struct pool *p) {
+
   /* Sanity check. */
-  if (! p)
+  if (!p)
     return;
 
-  block_alarms();
+  pr_alarms_block();
 
+  /* Run through any cleanups. */
   run_cleanups(p->cleanups);
   p->cleanups = NULL;
 
+  /* Destroy subpools. */
   while (p->sub_pools)
     destroy_pool(p->sub_pools);
   p->sub_pools = NULL;
@@ -372,39 +372,49 @@ static void clear_pool(struct pool *p)
   p->last = p->first;
   p->first->h.first_avail = p->free_first_avail;
 
-  unblock_alarms();
+  pr_alarms_unblock();
 }
 
-void destroy_pool(pool *p)
-{
-  block_alarms();
+void destroy_pool(pool *p) {
+  pr_alarms_block();
 
   if (p->parent) {
-    if (p->parent->sub_pools == p) p->parent->sub_pools = p->sub_next;
-    if (p->sub_prev) p->sub_prev->sub_next = p->sub_next;
-    if (p->sub_next) p->sub_next->sub_prev = p->sub_prev;
+    if (p->parent->sub_pools == p)
+      p->parent->sub_pools = p->sub_next;
+
+    if (p->sub_prev)
+      p->sub_prev->sub_next = p->sub_next;
+
+    if (p->sub_next)
+      p->sub_next->sub_prev = p->sub_prev;
   }
 
   clear_pool(p);
   free_blocks(p->first);
-
-  unblock_alarms();
+  pr_alarms_unblock();
 }
 
-long bytes_in_pool(pool *p) { return bytes_in_block_list(p->first); }
-long bytes_in_free_blocks(void) { return bytes_in_block_list(block_freelist); }
+#if 0
+/* NOTE: not used at the moment */
+static long bytes_in_pool(pool *p) {
+  return bytes_in_block_list(p->first);
+}
 
-/* Allocation inteface ... 
+static long bytes_in_free_blocks(void) {
+  return bytes_in_block_list(block_freelist);
+}
+#endif
+
+/* Allocation inteface ...
  */
 
-void *palloc(struct pool *p, int reqsize)
-{
-  /* Round up requested size to an even number of aligned units */
+void *palloc(struct pool *p, int reqsize) {
 
+  /* Round up requested size to an even number of aligned units */
   int nclicks = 1 + ((reqsize - 1) / CLICK_SZ);
   int size = nclicks * CLICK_SZ;
 
-  /* For performance, see if space is availabe in most recently
+  /* For performance, see if space is available in the most recently
    * allocated block.
    */
 
@@ -412,19 +422,18 @@ void *palloc(struct pool *p, int reqsize)
   char *first_avail = blok->h.first_avail;
   char *new_first_avail;
 
-  if(reqsize <= 0)
+  if (reqsize <= 0)
     return NULL;
 
-  block_alarms();
   new_first_avail = first_avail + size;
 
-  if(new_first_avail <= blok->h.endp) {
+  if (new_first_avail <= blok->h.endp) {
     blok->h.first_avail = new_first_avail;
-    unblock_alarms();
-    return (void *)first_avail;
+    return (void *) first_avail;
   }
 
   /* Need a new one that's big enough */
+  pr_alarms_block();
 
   blok = new_block(size);
   p->last->h.next = blok;
@@ -433,31 +442,31 @@ void *palloc(struct pool *p, int reqsize)
   first_avail = blok->h.first_avail;
   blok->h.first_avail += size;
 
-  unblock_alarms();
-  return (void*)first_avail;
+  pr_alarms_unblock();
+  return (void *) first_avail;
 }
 
-void *pcalloc(struct pool *p, int size)
-{
+void *pcalloc(struct pool *p, int size) {
   void *res = palloc(p, size);
   memset(res, '\0', size);
   return res;
 }
 
-char *pstrdup(struct pool *p, const char *s)
-{
+char *pstrdup(struct pool *p, const char *s) {
   char *res;
+  size_t len;
 
   if (!s)
     return NULL;
 
-  res = palloc(p, strlen(s) + 1);
-  sstrncpy(res, s, strlen(s) + 1);
+  len = strlen(s) + 1;
+
+  res = palloc(p, len);
+  sstrncpy(res, s, len);
   return res;
 }
 
-char *pstrndup(struct pool *p, const char *s, int n)
-{
+char *pstrndup(struct pool *p, const char *s, int n) {
   char *res;
 
   if (!s)
@@ -468,47 +477,46 @@ char *pstrndup(struct pool *p, const char *s, int n)
   return res;
 }
 
-char *pdircat(pool *p, ...)
-{
+char *pdircat(pool *p, ...) {
   char *argp, *res;
   char last;
 
   int len = 0, count = 0;
   va_list dummy;
 
-  va_start(dummy,p);
+  va_start(dummy, p);
 
   last = 0;
 
-  while((res = va_arg(dummy,char*)) != NULL) {
+  while ((res = va_arg(dummy, char *)) != NULL) {
     /* If the first argument is "", we have to account for a leading /
      * which must be added.  -jss 3/2/2001
      */
-    if(!count++ && !*res)
+    if (!count++ && !*res)
       len++;
-    else if(last && last != '/' && *res != '/')
+    else if (last && last != '/' && *res != '/')
       len++;
-    else if(last && last == '/' && *res == '/')
+    else if (last && last == '/' && *res == '/')
       len--;
     len += strlen(res);
-    last = res[strlen(res) - 1];
+    last = (*res ? res[strlen(res) - 1] : 0);
   }
 
   va_end(dummy);
   res = (char *) pcalloc(p, len + 1);
-  
+
   va_start(dummy, p);
-  
+
   last = 0;
-  
+
   while((argp = va_arg(dummy, char *)) != NULL) {
-    if(last && last == '/' && *argp == '/')
+    if (last && last == '/' && *argp == '/')
       argp++;
-    else if(last && last != '/' && *argp != '/')
+    else if (last && last != '/' && *argp != '/')
       sstrcat(res, "/", len + 1);
-    
+
     sstrcat(res, argp, len + 1);
-    last = res[strlen(res) - 1];
+    last = (*res ? res[strlen(res) - 1] : 0);
   }
 
   va_end(dummy);
@@ -516,29 +524,28 @@ char *pdircat(pool *p, ...)
   return res;
 }
 
-char *pstrcat(pool *p, ...)
-{
+char *pstrcat(pool *p, ...) {
   char *argp, *res;
 
-  int len = 0;
+  size_t len = 0;
   va_list dummy;
 
-  va_start(dummy,p);
-  
-  while((res = va_arg(dummy, char *)) != NULL)
+  va_start(dummy, p);
+
+  while ((res = va_arg(dummy, char *)) != NULL)
     len += strlen(res);
-  
+
   va_end(dummy);
-  
-  res = (char*) pcalloc(p, len + 1);
-  
-  va_start(dummy,p);
-  
-  while((argp = va_arg(dummy, char *)) != NULL)
+
+  res = (char *) pcalloc(p, len + 1);
+
+  va_start(dummy, p);
+
+  while ((argp = va_arg(dummy, char *)) != NULL)
     sstrcat(res, argp, len + 1);
-  
+
   va_end(dummy);
-  
+
   return res;
 }
 
@@ -546,11 +553,10 @@ char *pstrcat(pool *p, ...)
  * Array functions
  */
 
-array_header *make_array(pool *p, int nelts, int elt_size)
-{
-  array_header *res = (array_header*) palloc(p, sizeof(array_header));
+array_header *make_array(pool *p, int nelts, int elt_size) {
+  array_header *res = (array_header *) palloc(p, sizeof(array_header));
 
-  if(nelts < 1) nelts = 1;
+  if (nelts < 1) nelts = 1;
 
   res->elts = pcalloc(p, nelts * elt_size);
   res->pool = p;
@@ -561,9 +567,8 @@ array_header *make_array(pool *p, int nelts, int elt_size)
   return res;
 }
 
-void *push_array(array_header *arr)
-{
-  if(arr->nelts == arr->nalloc) {
+void *push_array(array_header *arr) {
+  if (arr->nelts == arr->nalloc) {
     char *new_data = pcalloc(arr->pool, arr->nalloc * arr->elt_size * 2);
 
     memcpy(new_data, arr->elts, arr->nalloc * arr->elt_size);
@@ -572,18 +577,18 @@ void *push_array(array_header *arr)
   }
 
   ++arr->nelts;
-  return ((char*)arr->elts) + (arr->elt_size * (arr->nelts - 1));
+  return ((char *)arr->elts) + (arr->elt_size * (arr->nelts - 1));
 }
 
 void array_cat(array_header *dst, const array_header *src)
 {
   int elt_size = dst->elt_size;
 
-  if(dst->nelts + src->nelts > dst->nalloc) {
+  if (dst->nelts + src->nelts > dst->nalloc) {
     int new_size = dst->nalloc * 2;
     char *new_data;
 
-    if(new_size == 0) ++new_size;
+    if (new_size == 0) ++new_size;
 
     while(dst->nelts + src->nelts > new_size)
       new_size *= 2;
@@ -595,7 +600,7 @@ void array_cat(array_header *dst, const array_header *src)
     dst->nalloc = new_size;
   }
 
-  memcpy(((char*)dst->elts) + dst->nelts * elt_size, (char*)src->elts,
+  memcpy(((char *)dst->elts) + dst->nelts * elt_size, (char *)src->elts,
          elt_size * src->nelts);
   dst->nelts += src->nelts;
 }
@@ -615,8 +620,8 @@ array_header *copy_array_str(pool *p, const array_header *arr)
   array_header *res = copy_array(p,arr);
   int i;
 
-  for(i = 0; i < arr->nelts; i++)
-    ((char**)res->elts)[i] = pstrdup(p,((char**)res->elts)[i]);
+  for (i = 0; i < arr->nelts; i++)
+    ((char **)res->elts)[i] = pstrdup(p, ((char **)res->elts)[i]);
 
   return res;
 }
@@ -689,7 +694,7 @@ void unregister_cleanup(pool *p, void *data, void (*cleanup_cb)(void *)) {
 /* NOTE: unused. */
 #if 0
 void run_cleanup(pool *p, void *data, void (*cleanup_cb)(void *)) {
-  block_alarms();
+  pr_alarms_block();
 
   /* Run the given cleanup callback. */
   (*cleanup_cb)(data);
@@ -697,7 +702,7 @@ void run_cleanup(pool *p, void *data, void (*cleanup_cb)(void *)) {
   /* Remove it. */
   unregister_cleanup(p, data, cleanup_cb);
 
-  unblock_alarms();
+  pr_alarms_unblock();
 }
 #endif
 
@@ -723,14 +728,14 @@ static void cleanup_pool_for_exec(pool *p) {
   run_child_cleanups(p->cleanups);
   p->cleanups = NULL;
 
-  for(p = p->sub_pools; p; p = p->sub_next)
+  for (p = p->sub_pools; p; p = p->sub_next)
     cleanup_pool_for_exec(p);
 }
 
 void cleanup_for_exec(void) {
-  block_alarms();
+  pr_alarms_block();
   cleanup_pool_for_exec(permanent_pool);
-  unblock_alarms();
+  pr_alarms_unblock();
 }
 #endif
 
@@ -749,20 +754,20 @@ static void register_fd_cleanups(pool *p, int fd) {
 int popenf(pool *p, const char *name, int flags, int mode) {
   int fd;
 
-  block_alarms();
+  pr_alarms_block();
   if ((fd = open(name, flags, mode)) >= 0)
     register_fd_cleanups(p, fd);
-  unblock_alarms();
+  pr_alarms_unblock();
   return fd;
 }
 
 int pclosef(pool *p, int fd) {
   int res;
 
-  block_alarms();
+  pr_alarms_block();
   res = close(fd);
   unregister_cleanup(p, (void *)fd, fd_cleanup_cb);
-  unblock_alarms();
+  pr_alarms_unblock();
   return res;
 }
 
@@ -786,7 +791,7 @@ FILE *pfopen(pool *p, const char *name, const char *mode) {
   FILE *fd = NULL;
   int base_flag, desc;
 
-  block_alarms();
+  pr_alarms_block();
 
   if (*mode == 'a') {
     base_flag = (*(mode+1) == '+') ? O_RDWR : O_WRONLY;
@@ -801,27 +806,27 @@ FILE *pfopen(pool *p, const char *name, const char *mode) {
   if (fd)
     register_file_cleanups(p, fd);
 
-  unblock_alarms();
+  pr_alarms_unblock();
   return fd;
 }
 
 FILE *pfdopen(pool *p, int fd, const char *mode) {
   FILE *f;
 
-  block_alarms();
+  pr_alarms_block();
   if ((f = fdopen(fd, mode)) != NULL)
     register_file_cleanups(p, f);
 
-  unblock_alarms();
+  pr_alarms_unblock();
   return f;
 }
 
 int pfclose(pool *p, FILE *fd) {
   int res;
 
-  block_alarms();
+  pr_alarms_block();
   res = fclose(fd);
   unregister_cleanup(p, (void *) fd, file_cleanup_cb);
-  unblock_alarms();
+  pr_alarms_unblock();
   return res;
 }
